//
//  BBSAppAppDelegate.h
//  BBSApp
//
//  Created by Ki Woon Sung on 7/25/08.
//  Copyright Columbia University 2008. All rights reserved.
//

#import <UIKit/UIKit.h>

#import "BonjourMonitor.h"

@class BBSAppViewController;
@class HTTPServer;

@interface BBSAppAppDelegate : NSObject <UIApplicationDelegate> {
	IBOutlet UIWindow *window;
	UITabBarController *tabBarController;
    
    // Information shared among all subviews and controllers
    NSString *_serverIPv4Address;
    NSString *_serverListeningPort;
    NSString *_domainName; // for service discovery
    NSString *_serviceType; // for service discovery
    NSString *_defaultRepositoryName; // the dedicated name for the 7DS BBS applications both for Java/iPhone
    NSString *_registeredName; // the name used when this device is registered to ZeroConf
    NSString *_localIPv4Address;
    NSString *_localListeningPort;
    
    // Bonjour service
    BonjourMonitor *_bonjourMonitor;
    NSString *serverURLString; // Contains the URL including port to BBS_REP
    NSString *selfURLString;
    NSString *serverHostNameString;
    NSString *selfRegisteredNameString;
    
    NSNetService *bbsNetService;
    NSUInteger port;
    
    // HTTP server
    HTTPServer *httpServer;
    
    // Flag to see if it's okay to upload
    BOOL isServerFound;
    BOOL isPublished;
    NSNetService *serverNetService;
}

@property (nonatomic, retain) UIWindow *window;
@property (nonatomic, retain) UITabBarController *tabBarController;

@property (nonatomic, retain) NSString *serverIPv4Address;
@property (nonatomic, retain) NSString *serverListeningPort;
@property (nonatomic, retain) NSString *domainName;
@property (nonatomic, retain) NSString *serviceType;
@property (nonatomic, retain) NSString *defaultRepositoryName;
@property (nonatomic, retain) NSString *registeredName;
@property (nonatomic, retain) NSString *localIPv4Address;
@property (nonatomic, retain) NSString *localListeningPort;

@property (nonatomic, retain) BonjourMonitor *bonjourMonitor;
@property (nonatomic, retain) NSString *serverURLString;
@property (nonatomic, retain) NSString *selfURLString;
@property (nonatomic, retain) NSString *serverHostNameString;

@property (nonatomic, retain) NSNetService *bbsNetService;
@property (nonatomic, retain) HTTPServer *httpServer;
@property (nonatomic, retain) NSString *selfRegisteredNameString;
@property (nonatomic, retain) NSNetService *serverNetService;
@property BOOL isServerFound;


//[self setNetworkInformation]; // aviailability/service name and so forth
//[self setRepositoryInformation]; // dedicated name for the repository/its ip n port/and so forth
//[self setLocalInformation];  // local ip/port/service name and so forth
//

- (void)setupUserInterface;
- (void)setNetworkInformation;
- (void)setRepositoryInformation;
- (void)setLocalInformation;
- (NSString *)getServerIPv4Address;
- (NSString *)getServerListeningPort;
- (NSString *)getServerURLString;
- (void)startBonjourMonitor;
- (void)stopBonjourMonitor;
- (void)setServerConnectionInformation:(NSString *)URLString;
- (void)setSelfConnectionInformation:(NSString *)URLString;
- (void)setSelfRegisteredName:(NSString *)newName;
- (NSString *)getSelfRegisteredName;
- (NSString *)getServerConnectionInformation;
- (NSString *)getSelfConnectionInformation;
- (void)setServerHostName:(NSString *)hostNameString;
- (NSString *)getServerHostName;

// See if it is okay to upload
- (void)setDidPublished;
- (void)tryUploadAllItems;
- (void)setServerFound:(NSNetService *)_serverNetService;
- (void)uploadLocalItems:(NSNetService *)_serverNetService;

// publishing: not in use
- (void)publishBBSServiceOnPort:(NSUInteger)newPort;

// Internal web server
// Launch the internal http server and publish the related information to zeroconf network
- (void)launchHTTPDAndPublishItems;
- (void)runService;

// Development purpose
- (void)deleteXMLsInLocal;

// Initialize sub-folders to store items
- (void)setupSubfolders;
- (void)moveDefaultItemFilesToPublicFolder;

@end

