//
//  BBSAppAppDelegate.m
//  BBSApp
//
//  Created by Ki Woon Sung on 7/25/08.
//  Copyright Columbia University 2008. All rights reserved.
//

#import "BBSAppAppDelegate.h"

#import "Contants.h"
// View controllers in the navigation bar
#import "ServiceTableViewController.h"
#import "PeerItemTableViewController.h"
#import "ServerItemTableViewController.h"
#import "ServerItemTableViewControllerR2.h"
#import "LocalItemTableViewController.h"
#import "ComposerTableViewController.h"
#import "AboutViewController.h"
#import "ItemTableViewControllerTest.h"
#import "ButtonsViewController.h"

// Data models and handles
#import "Item.h"
#import "ServerInformationHandler.h"

#import "BonjourMonitor.h"

#import "BBSPublisher.h"
#import "HTTPLauncher.h"
#import "HTTPServer.h"

#import "LocalFeedGenerator.h"

@implementation BBSAppAppDelegate

@synthesize window;
@synthesize tabBarController;

@synthesize serverIPv4Address = _serverIPv4Address;
@synthesize serverListeningPort = _serverListeningPort;
@synthesize domainName = _domainName;
@synthesize serviceType = _serviceType;
@synthesize defaultRepositoryName = _defaultRepositoryName;
@synthesize registeredName = _registeredName;
@synthesize localIPv4Address = _localIPv4Address;
@synthesize localListeningPort = _localListeningPort;

@synthesize bonjourMonitor = _bonjourMonitor;
@synthesize serverURLString;
@synthesize selfURLString;
@synthesize serverHostNameString;
@synthesize selfRegisteredNameString;

@synthesize bbsNetService;
@synthesize httpServer;

@synthesize serverNetService;
@synthesize isServerFound;
- (id) init
{
	self = [super init];
	if (self != nil) {
        self.isServerFound = NO;
        isPublished = NO;
        self.serverNetService = nil;
        
		// initialize to nil
		window = nil;
		tabBarController = nil;
        
        // Initialize the general information to run the 7DS BBS application
        // Set the information needed to run this program
        [self setNetworkInformation]; // aviailability/service name and so forth
        [self setRepositoryInformation]; // dedicated name for the repository/its ip n port/and so forth
        [self setLocalInformation];  // local ip/port/service name and so forth
        [self startBonjourMonitor];

        [self launchHTTPDAndPublishItems];
	}
	return self;
}


- (void)applicationDidFinishLaunching:(UIApplication *)application {	
	
	// configure the portrait user interface
	[self setupUserInterface];
    
    // Publish local items to Zeroconf network
//    [self publishBBSItems];
    
    // For developing purpose : call the function from the covert screen
//    [self deleteXMLsInLocal];
    
    // initialize sub folders
    [self setupSubfolders];
}


- (void)dealloc {
	[tabBarController release];
	[window release];
	[super dealloc];
}



- (void)setupUserInterface {
	// a local navigation variable
	// this is reused several times
	UINavigationController *localNavigationController;
	
    // Set up the portraitWindow and content view
	UIWindow *localPortraitWindow;
	localPortraitWindow = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window = localPortraitWindow;
	
	// the localPortraitWindow data is now retained by the application delegate
	// so we can release the local variable
	[localPortraitWindow release];
	
	
    [window setBackgroundColor:[UIColor blackColor]];
    
	// Create a tabbar controller and an array to contain the view controllers
	tabBarController = [[UITabBarController alloc] init];
	NSMutableArray *localViewControllersArray = [[NSMutableArray alloc] initWithCapacity:5];

	// 1. 
	// Create the table view controller for the server item list
	ServiceTableViewController *serviceTableViewController;	
	serviceTableViewController = [[ServiceTableViewController alloc] initWithStyle:UITableViewStylePlain];
		
	localNavigationController = [[UINavigationController alloc] initWithRootViewController:serviceTableViewController];
    localNavigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[localViewControllersArray addObject:localNavigationController];
	
	// the localNavigationController data is now retained by the application delegate
	// so we can release the local variable
	[serviceTableViewController release];
	[localNavigationController release];
		
	// 2.
	// Create the table view controller for the server item list
    // NOTE: This only calls ServerItemTableViewController revision 2.
    // R1 was deprecated since it doesn't support toolbar in a convenient way.
    // Careful: R2 is based on UIView. Initiate it in a appropriate way.
	ServerItemTableViewControllerR2 *serverItemTableViewController;	
	serverItemTableViewController = [[ServerItemTableViewControllerR2 alloc] initWithNibName:nil bundle:nil];
	
	localNavigationController = [[UINavigationController alloc] initWithRootViewController:serverItemTableViewController];
    localNavigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[localViewControllersArray addObject:localNavigationController];
	
	// the localNavigationController data is now retained by the application delegate
	// so we can release the local variable
	[serverItemTableViewController release];
	[localNavigationController release];
	
	// 3.
	// Create the table view controller for the my item (local) list
	LocalItemTableViewController *localItemTableViewController;	
	localItemTableViewController = [[LocalItemTableViewController alloc] initWithStyle:UITableViewStylePlain];
	
	localNavigationController = [[UINavigationController alloc] initWithRootViewController:localItemTableViewController];
    localNavigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[localViewControllersArray addObject:localNavigationController];
	
	// the localNavigationController data is now retained by the application delegate
	// so we can release the local variable
	[localItemTableViewController release];
	[localNavigationController release];
	
    /* On the demonstration, the following controllers don't need to be showed up
       on the main screen
    /* Demo block: begin */
	// 4.
	// Create the table view controller for the composer screen
	ComposerTableViewController *composerTableViewController;	
	composerTableViewController = [[ComposerTableViewController alloc] initWithStyle:UITableViewStylePlain];
	
	localNavigationController = [[UINavigationController alloc] initWithRootViewController:composerTableViewController];
	[localViewControllersArray addObject:localNavigationController];
    
    [composerTableViewController release];
	[localNavigationController release];
	
	// 5.
	// Create the view controller for About screen
#define ABOUT_WINDOW
#ifndef ABOUT_WINDOW
	AboutViewController *aboutViewController;	
	aboutViewController = [[AboutViewController alloc] initWithNibName:nil bundle:nil];

    localNavigationController = [[UINavigationController alloc] initWithRootViewController:aboutViewController];
	[localViewControllersArray addObject:localNavigationController];

    [aboutViewController release];
	[localNavigationController release];
#endif
    
    // 6. this is for testing: ItemTableViewControllerTest
#define TEST_WINDOW
#ifdef TEST_WINDOW
	ItemTableViewControllerTest *localItemTableViewControllerTest;	
	localItemTableViewControllerTest = [[ItemTableViewControllerTest alloc] initWithNibName:nil bundle:nil];
	
	localNavigationController = [[UINavigationController alloc] initWithRootViewController:localItemTableViewControllerTest];
    localNavigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[localViewControllersArray addObject:localNavigationController];
	
	// the localNavigationController data is now retained by the application delegate
	// so we can release the local variable
	[localItemTableViewControllerTest release];
	[localNavigationController release];
#endif
    // end.6.
    
    // 7. This is a temporary table view controller to modify ServerItemTableViewController R1.
     /*ServerItemTableViewControllerR2 *localServerItemTableViewControllerR2;	
     localServerItemTableViewControllerR2 = [[ServerItemTableViewControllerR2 alloc] initWithNibName:nil bundle:nil];
     
     localNavigationController = [[UINavigationController alloc] initWithRootViewController:localServerItemTableViewControllerR2];
     localNavigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
     [localViewControllersArray addObject:localNavigationController];
     
     // the localNavigationController data is now retained by the application delegate
     // so we can release the local variable
     [localServerItemTableViewControllerR2 release];
     [localNavigationController release];
    // end.7
    
    
    // 8. Covert UI for testing and experiments
	ButtonsViewController *buttonsViewController;	
	buttonsViewController = [[ButtonsViewController alloc] initWithNibName:nil bundle:nil];
    
    localNavigationController = [[UINavigationController alloc] initWithRootViewController:buttonsViewController];
	[localViewControllersArray addObject:localNavigationController];
    
    [buttonsViewController release];
	[localNavigationController release];
    
    */ // Demo block end
	
    
	// set the tab bar controller view controller array to the localViewControllersArray
	tabBarController.viewControllers = localViewControllersArray;
	
	// the localViewControllersArray data is now retained by the tabBarController
	// so we can release this version
	[localViewControllersArray release];
	
	// set the window subview as the tab bar controller
	[window addSubview:tabBarController.view];
	
	// make the window visible
	[window makeKeyAndVisible];
	
	
}



- (void)reloadTable
{
    //[[(ServerItemTableViewController *)[self.navigationController topViewController] tableView] reloadData];
}


- (void)addToItemList:(Item *)newItem {
//    [self.list addObject:newItem];
    // The XML parser calls addToItemList: each time it creates an item object.
    // The table needs to be reloaded to reflect the new content of the list.
//    [self reloadTable];
    NSLog(@"addToItemList called");
}



//[self setNetworkInformation]; // aviailability/service name and so forth
//[self setRepositoryInformation]; // dedicated name for the repository/its ip n port/and so forth
//[self setLocalInformation];  // local ip/port/service name and so forth
//

- (void)setNetworkInformation {
    // TEMPORARY ASSIGNMENTS
    self.localIPv4Address = @"192.168.1.4";
    self.domainName = @"local";
    self.serviceType = @"7ds_bbs._tcp";
}

- (void)setRepositoryInformation {
    ServerInformationHandler *sih = [[ServerInformationHandler alloc]init];
    self.defaultRepositoryName = [sih getRepositoryName];
    self.serverIPv4Address = [sih getServerIPv4Address];
    self.serverListeningPort = [sih getServerListeningPort];
    
    [sih release];
}


- (void)setLocalInformation {
    self.localIPv4Address = @"192.168.1.4";
    self.localListeningPort = @"80";
    
}


- (NSString *)getServerIPv4Address {
    return self.serverIPv4Address;
}


- (NSString *)getServerListeningPort {
    return self.serverListeningPort;
}


- (NSString *)getServerURLString {
    NSString *serverIPv4Address = [self getServerIPv4Address];
    NSString *serverListeningPort = [self getServerListeningPort];
    
    NSString *URLString = [@"http://" stringByAppendingString:serverIPv4Address];
    URLString = [URLString stringByAppendingString:@":"];
    URLString = [URLString stringByAppendingString:serverListeningPort];
    URLString = [URLString stringByAppendingString:@"/"];
    //URLString = [URLString stringByAppendingString:fileName];

    NSLog(@"server information is requested: %@", URLString);
    
    return URLString;
}


- (void)startBonjourMonitor {
    // This is better to be called after the view of the peer item list is loaded.
    // in order to show the progress status on the peer item list which is the first
    // screen of the application
    self.bonjourMonitor = [[BonjourMonitor alloc]init];
    [self.bonjourMonitor startMonitoring];
}


- (void)stopBonjourMonitor {
   //  NSLog(@"stopBonjour");
    [self.bonjourMonitor stopMonitoring];
}


- (void)setSelfRegisteredName:(NSString *)newName {
    self.selfRegisteredNameString = newName;
}


- (NSString *)getSelfRegisteredName {
    return self.selfRegisteredNameString;
}


- (void)setServerConnectionInformation:(NSString *)URLString {
    self.serverURLString = [[NSString alloc]initWithString:URLString];
    NSLog(@"BBS_REP information assigned");
}


- (void)setSelfConnectionInformation:(NSString *)URLString {
    self.selfURLString = [[NSString alloc]initWithString:URLString];
    NSLog(@"self information assigned");
}


- (void)setServerHostName:(NSString *)hostNameString {
    self.serverHostNameString = [[NSString alloc]initWithString:hostNameString];
}


- (NSString *)getServerConnectionInformation {
    NSLog(@"server information returned");
    return self.serverURLString;
}


- (NSString *)getSelfConnectionInformation {
    return self.selfURLString;
}


- (NSString *)getServerHostName {
    return self.serverHostNameString;
}


// deprecated
- (void)publishBBSServiceOnPort:(NSUInteger)newPort {
    BBSPublisher *bbsPublisher = [[BBSPublisher alloc]initWithNewPort:newPort];
    
    // Publish the service
    if ([bbsPublisher enableBonjour]) {
        NSLog(@"service published: yes");
    } else {
        NSLog(@"service NOT published");
    }           
}

// launch http daemon and publish related information to zeroconf
- (void)launchHTTPDAndPublishItems {
    // Deprecated class:
    // But sometimes it is unblocked to be used as a covert channel
    HTTPLauncher *httpd = [[HTTPLauncher alloc]initWithPort:@"20001"];
    [httpd launchHTTPDaemon];
    
    // Launch HTTPServer class as the main internal web server
    // It also publishes local items to Zeroconf
    //	[httpServer setType:@"_http._tcp."];

    self.httpServer = [[HTTPServer alloc] init];
    
    //[NSThread detachNewThreadSelector:@selector(runService:) toTarget:self.httpServer withObject:nil];
    [self runService];
}



- (void)runService 
{
    //kwsung
    [self.httpServer setType:@"_7dsbbs._tcp"];
    NSString *fullPath = [[NSBundle mainBundle] bundlePath];
    [self.httpServer setDocumentRoot:[NSURL fileURLWithPath:fullPath]];// stringByExpandingTildeInPath]]];
    
    // Get the newly assigned port number
    port = [self.httpServer port];
    
    NSError *error;
    BOOL success = [self.httpServer start:&error];
    
    if(!success)
    {
        NSLog(@"Error starting HTTP Server: %@", error);
    } else {
        NSLog(@"HTTPServer started");
    }
}


- (void)deleteXMLsInLocal {
    // Keep this code for the future reference
    NSDirectoryEnumerator *direnum = [[NSFileManager defaultManager] 
                                      enumeratorAtPath:NSHomeDirectory()]; //stringByAppendingString:@"/Documents"]]; 
    NSString *pname; 
    NSString *fileName = @"";
    NSError **error = nil;
    
    while (pname = [direnum nextObject]) 
    { 
        if ([[pname pathExtension] isEqualToString:@"rtfd"]) 
        { 
            /* don’t enumerate this directory */ 
            [direnum skipDescendents]; 
        } 
        else if ([[pname pathExtension] isEqualToString:@"xml"]) 
        { 
            /* ...process file here... */ 
            fileName = [@"/" stringByAppendingString:[pname lastPathComponent]];
            
            NSString *fullPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:fileName];
            NSLog(@"fullPath of XML file:%@", fullPath);
            
            // Delete
            if([[NSFileManager defaultManager]removeItemAtPath:fullPath error:error]) {
                NSLog(@"deleted:%@", fullPath);
            } else {
                NSLog(@"Fail to delete:%@", fullPath);
            }
        }
    }
}


- (void)setDidPublished {
    isPublished = YES;
}



- (void)setServerFound:(NSNetService *)_serverNetService {
    isServerFound = YES;
    self.serverNetService = _serverNetService;
}


- (void)tryUploadAllItems {
    if ((isPublished) && (isServerFound)&&(self.serverNetService != nil)) {
        NSLog(@"start uploading");
        [self uploadLocalItems:self.serverNetService];
    }
}


- (void)uploadLocalItems:(NSNetService *)_serverNetService {
    BBSAppAppDelegate *appDelegate = [[UIApplication sharedApplication]delegate];
    NSDirectoryEnumerator *direnum = [[NSFileManager defaultManager] 
                                      enumeratorAtPath:NSHomeDirectory()];
    NSString *pname; 
    NSString *fileName = @"";
    NSURLConnection *theConnection;
    while (pname = [direnum nextObject]) 
    { 
        if ([[pname pathExtension] isEqualToString:@"rtfd"]) { 
        } 
        else if ([[pname pathExtension] isEqualToString:@"xml"]) 
        { 
            fileName = [@"/" stringByAppendingString:[pname lastPathComponent]];
            
            NSString *URLString = @"";
            
            URLString = [URLString stringByAppendingFormat:@"http://%@:%d/", [_serverNetService hostName], [_serverNetService port]];
            NSLog(@"try to UPLOAD all local items to \"%@\"", URLString);
            
            // Adding percent escapes
            URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
            NSLog(@"URL request after percent sequence adding: %@", URLString);
            
            NSString *fullPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:fileName];
            NSLog(@"fullPath of XML file:%@", fullPath);
            
            NSString *post = @"registered_name=";
            post = [post stringByAppendingString:[appDelegate getSelfRegisteredName]];
            post = [post stringByAppendingString:@"&itemName="];
            post = [post stringByAppendingString:[[fileName lastPathComponent] stringByDeletingPathExtension]];
            post = [post stringByAppendingString:@"&count=1&0="];
            
            NSData *fileData = [[NSFileManager defaultManager]contentsAtPath:fullPath];
            NSString *fileDataString = [[NSString alloc]initWithData:fileData encoding:NSUTF8StringEncoding];
            post = [post stringByAppendingString:fileDataString];
            NSLog(@"post request=%@", post);
            
            NSData *postData = [post dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
            
            NSString *postLength = [NSString stringWithFormat:@"%d", [postData length]];
            
            NSMutableURLRequest *request = [[[NSMutableURLRequest alloc] init] autorelease];
//            URLString = [URLString stringByAppendingString:@"BBS"]; // no when uploading
            URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
            [request setURL:[NSURL URLWithString:URLString]];
            [request setHTTPMethod:@"POST"];
            [request setValue:postLength forHTTPHeaderField:@"Content-Length"];
            [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
            [request setHTTPBody:postData];
            [request setTimeoutInterval:_REQUEST_TIME_OUT_IN_SEC];
            [request setCachePolicy:NSURLRequestUseProtocolCachePolicy];
            
            // Asynchronous send
            theConnection=[[NSURLConnection alloc] initWithRequest:request 
                                                          delegate:self]; 
            
        }
    }
    if (theConnection) { 
        NSLog(@"sent: success");
    } else { 
        // inform the user that the download could not be made 
        NSLog(@"error on requesting");
    } 
}


// create the default sub-folders if not exists
- (void)setupSubfolders {
    // public/private/RFU
    NSLog(@"setupSubfolders");

    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *documentsPath = @"";
    documentsPath = [documentsPath stringByAppendingString:NSHomeDirectory()];
    documentsPath = [documentsPath stringByAppendingString:@"/Documents"];
    NSLog(@"documentsPath = \"%@\"", documentsPath);
    BOOL isDir;

    if ([fileManager fileExistsAtPath:documentsPath isDirectory:&isDir] && YES) {
        NSLog(@"documentsPath exists");
        
        NSString *publicFolderPath = @"";
        NSString *privateFolderPath = @"";
        NSString *draftFolderPath = @"";
        NSString *rfuFolderPath = @"";
        
        publicFolderPath = [documentsPath stringByAppendingString:@"/public"];
        privateFolderPath = [documentsPath stringByAppendingString:@"/private"];
        draftFolderPath = [documentsPath stringByAppendingString:@"/draft"];
        rfuFolderPath = [documentsPath stringByAppendingString:@"/rfu"];
        
        NSLog(@"public path : %@", publicFolderPath);
        NSLog(@"private path: %@", privateFolderPath);
        NSLog(@"draft path  : %@", draftFolderPath);
        NSLog(@"rfu path    : %@", rfuFolderPath);
        
        if (![fileManager fileExistsAtPath:publicFolderPath isDirectory:&isDir]) {
            [fileManager createDirectoryAtPath:publicFolderPath attributes:nil];
            NSLog(@"public folder created");
        }
        
        if (![fileManager fileExistsAtPath:privateFolderPath isDirectory:&isDir]) {
            [fileManager createDirectoryAtPath:privateFolderPath attributes:nil];
            NSLog(@"private folder created");
        }
        
        if (![fileManager fileExistsAtPath:draftFolderPath isDirectory:&isDir]) {
            [fileManager createDirectoryAtPath:draftFolderPath attributes:nil];
            NSLog(@"private folder created");
        }
        
        if (![fileManager fileExistsAtPath:rfuFolderPath isDirectory:&isDir]) {
            [fileManager createDirectoryAtPath:rfuFolderPath attributes:nil];
            NSLog(@"rfu folder created");
        }
        
        // Move default item XML files if exists in the home directory
        [self moveDefaultItemFilesToPublicFolder];
        
        // Generate a form feed file for local item listing
        LocalFeedGenerator *localFeedGenerator = [[LocalFeedGenerator alloc] init];
        [localFeedGenerator generateLocalFeed];
        
    } else {
        NSLog(@"NO: %@", documentsPath);
    }
    
    
    
//    - (BOOL)createDirectoryAtPath:(NSString *)path attributes:(NSDictionary *)attributes
}


- (void) moveDefaultItemFilesToPublicFolder {
    NSLog(@"moveDefaultItemFilesToPublicFolder");
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    NSString *defaultFileName = @"";
    NSString *fullPath = [[NSBundle mainBundle] bundlePath];
    NSLog(@"fullPath = %@", fullPath);
    
    defaultFileName = [defaultFileName stringByAppendingString:fullPath];
    defaultFileName = [defaultFileName stringByAppendingString:@"/default_01.xml"];
    
    NSString *destFileName = @"";
    destFileName = [destFileName stringByAppendingString:NSHomeDirectory()];
    destFileName = [destFileName stringByAppendingString:@"/Documents/public/default_01.xml"];
    
    NSString *destFileName2 = @"";
    destFileName2 = [destFileName2 stringByAppendingString:NSHomeDirectory()];
    destFileName2 = [destFileName2 stringByAppendingString:@"/Documents/public/default_02.xml"];

    NSLog(@"defaultFileName = \"%@\"", defaultFileName);
    NSLog(@"destFileName = \"%@\"", destFileName);
    
    BOOL isDir;
    
    if ([fileManager fileExistsAtPath:defaultFileName isDirectory:&isDir]) {
        [fileManager moveItemAtPath:defaultFileName toPath:destFileName error:nil];
//        [fileManager moveItemAtPath:defaultFileName toPath:destFileName handler:nil];
        [fileManager moveItemAtPath:defaultFileName toPath:destFileName2 error:nil];
//        [fileManager movePath:defaultFileName toPath:destFileName2 handler:nil];
        NSLog(@"default file has been moved to the draft folder");
    }
    
}


@end
