//
//  BBSPublisher.m
//  BBSApp
//
//  Created by Kiwoon Sung on 8/5/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#include <CFNetwork/CFSocketStream.h>

#import "BBSPublisher.h"

#import <netinet/in.h>
#import <sys/socket.h>
#include <unistd.h>


@implementation BBSPublisher


@synthesize netService = _netService;
@synthesize port = _port;
//@synthesize socket;

- (id)initWithNewPort:(NSUInteger)newPort {
    self = [super init];
    if (self != nil) {
        self.port = newPort;
    }
    return self;
}



// Not in use anymore
- (BOOL)initSocket {    
    NSError **error = nil;
    
    CFSocketContext socketCtxt = {0, self, NULL, NULL, NULL};
    _ipv4socket = CFSocketCreate(kCFAllocatorDefault, PF_INET, SOCK_STREAM, IPPROTO_TCP, 0, nil, &socketCtxt);
	
    if (NULL == _ipv4socket) {
        if (error) *error = [[NSError alloc] initWithDomain:@"TCPServerErrorDomain" code:kTCPServerNoSocketsAvailable userInfo:nil];
        if (_ipv4socket) CFRelease(_ipv4socket);
        _ipv4socket = NULL;
        return NO;
    }
	
	
    int yes = 1;
    setsockopt(CFSocketGetNative(_ipv4socket), SOL_SOCKET, SO_REUSEADDR, (void *)&yes, sizeof(yes));
	
    // set up the IPv4 endpoint; use port 0, so the kernel will choose an arbitrary port for us, which will be advertised using Bonjour
    struct sockaddr_in addr4;
    memset(&addr4, 0, sizeof(addr4));
    addr4.sin_len = sizeof(addr4);
    addr4.sin_family = AF_INET;
    addr4.sin_port = 0;
    addr4.sin_addr.s_addr = htonl(INADDR_ANY);
    NSData *address4 = [NSData dataWithBytes:&addr4 length:sizeof(addr4)];
	
    if (kCFSocketSuccess != CFSocketSetAddress(_ipv4socket, (CFDataRef)address4)) {
        if (error) *error = [[NSError alloc] initWithDomain:@"TCPServerErrorDomain" code:kTCPServerCouldNotBindToIPv4Address userInfo:nil];
        if (_ipv4socket) CFRelease(_ipv4socket);
        _ipv4socket = NULL;
        return NO;
    }
    
	// now that the binding was successful, we get the port number 
	// -- we will need it for the NSNetService
	NSData *addr = [(NSData *)CFSocketCopyAddress(_ipv4socket) autorelease];
	memcpy(&addr4, [addr bytes], [addr length]);
	self.port = ntohs(addr4.sin_port);
	
	
    // set up the run loop sources for the sockets
    CFRunLoopRef cfrl = CFRunLoopGetCurrent();
    CFRunLoopSourceRef source4 = CFSocketCreateRunLoopSource(kCFAllocatorDefault, _ipv4socket, 0);
    CFRunLoopAddSource(cfrl, source4, kCFRunLoopCommonModes);
    CFRelease(source4);

    // Free the socket
    if (_ipv4socket) {
		CFSocketInvalidate(_ipv4socket);
		CFRelease(_ipv4socket);
		_ipv4socket = NULL;
	}
	
    NSLog(@"new port = %d", self.port);
	
    return YES;
}

- (void)testSocket {

}


- (BOOL)enableBonjour {
	self.netService = [[NSNetService alloc] initWithDomain:@"" type:@"_7dsbbs._tcp" name:@"" port:self.port];
	if(self.netService == nil)
		return NO;
	
	[self.netService scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[self.netService publish];
	[self.netService setDelegate:self];
	
	return YES;
}

    
//- (void)initSocket {    
//    //    NSSocketPort *aSocket = [[NSSocketPort alloc] init];// 1
//    //    self.socket = aSocket;
//    //    [aSocket release];
//    //
//    //    if(self.socket)
//    //    {
//    //        self.service = [[NSNetService alloc] initWithDomain:@""// 4
//    //                                                  type:@"_7dsbbs._tcp"
//    //                                                  name:@"" port:20001];
//    //        
//    //        if(self.service)
//    //        {
//    //            NSLog(@"service published: SocketHandlingTest.m");
//    //            NSLog(@"port=%d", [self.service port]);
//    //            [self.service setDelegate:self];// 5
//    //            [self.service publish];// 6
//    //        }
//    //        else
//    //        {
//    //            NSLog(@"An error occurred initializing the NSNetService object.");
//    //        }
//    //    }
//    //    else
//    //    {
//    //        NSLog(@"An error occurred initializing the NSSocketPort object.");
//    //    }
//}
    
@end
