//
//  BonjourMonitor.m
//  BBSApp
//
//  Created by Kiwoon Sung on 8/2/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import "Contants.h"
#import "BonjourMonitor.h"

#import "BBSAppAppDelegate.h"


@implementation BonjourMonitor

@synthesize netServiceBrowser = _netServiceBrowser;
@synthesize netService = _netService;
@synthesize netServices = _netServices;
@synthesize listItemInformation = _listItemInformation;

- (id) init
{
    self = [super init];
    if (self != nil) {
        self.netServices = [[NSMutableArray alloc]init];
        searching = NO;
        
        NSNetServiceBrowser *aNetServiceBrowser = [[NSNetServiceBrowser alloc]init];
        [aNetServiceBrowser setDelegate:self];
        self.netServiceBrowser = aNetServiceBrowser;
        [aNetServiceBrowser release];

    }
    return self;
}


- (void)dealloc {
    [self.netServices release];
    [super dealloc];
}

    

#pragma mark NSNetServiceBrowser delegate methods

- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didFindDomain:(NSString*)domain moreComing:(BOOL)moreComing {
//	NSString* tmp = [self transmogrify:domain];
//	if (![self.domains containsObject:tmp]) [self.domains addObject:tmp];
    if (moreComing) NSLog(@"moreComing");
    
	// moreComing really means that there are no more messages in the queue from the BonjourMonitor daemon, so we should update the UI.
	// When moreComing is set, we don't update the UI so that it doesn't 'flash'.
//	if (!moreComing)
//		[self updateUI];
}	


- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didFindService:(NSNetService *)netService moreComing:(BOOL)moreServicesComing {
    if ([[netService name] isEqualToString:@"BBS_REP"]) {
        NSLog(@"central repository found");
        self.netService = netService;
        [self.netService setDelegate:self];
        [self.netService resolve];
    }
    
    if (moreServicesComing) NSLog(@"more coming");
}


- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didRemoveService:(NSNetService *)netService moreComing:(BOOL)moreServicesComing {
    NSLog(@"didRemoveService:%@", [netService name]);
    
    if (!moreServicesComing) {
        NSLog(@"No more coming");
    }
}



- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didNotSearch:(NSDictionary *)errorInfo {
    NSLog(@"didNotSearch");
    
    searching = NO;
    [self handleError:[errorInfo objectForKey:NSNetServicesErrorCode]];
}


- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didRemoveDomain:(NSString*)domain moreComing:(BOOL)moreComing {
    NSLog(@"netServiceBrowser:netServiceBrowser didRemoveDomain: moreComing");

    if (moreComing)
        NSLog(@"moreComing");
}	


- (void)netServiceBrowserDidStopSearch:(NSNetServiceBrowser *)netServiceBrowser {
    NSLog(@"netServiceBrowserDidStopSearch");
    searching = NO;
}



- (void)netServiceBrowserWillSearch:(NSNetServiceBrowser *)netServiceBrowser {
    searching = YES;
    NSLog(@"netServiceBrowserWillSearch");
}


// NSNetService delegate methods
#pragma mark NSNetService delegate methods


- (void)netService:(NSNetService *)sender didNotPublish:(NSDictionary *)errorDict {
    NSLog(@" didNotPublish");
    
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
}


- (void)netService:(NSNetService *)sender didNotResolve:(NSDictionary *)errorDict {
    NSLog(@" didNotResolve");
    
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
}


- (void)netServiceDidResolveAddress:(NSNetService *)sender {
    NSLog(@"------------------------------------------------------------");
    NSLog(@"netServiceDidResolveAddress");
    NSLog(@"\tname: %@", [sender name]);
    NSLog(@"\thostName: %@", [sender hostName]);
    NSLog(@"port: %d", [sender port]);
    NSLog(@"\tdomain : %@", [sender domain]);
    NSLog(@"\ttype : %@", [sender type]);
    
    if ([[sender name] isEqualToString:@"BBS_REP"]) {
        NSLog(@"BBS_REP address resolved");
        BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication]delegate];
        NSString *URLString = @"";
        URLString = [URLString stringByAppendingFormat:@"http://%@:%d/", [sender hostName], [sender port]];
        NSString *hostNameString = @"";
        hostNameString = [hostNameString stringByAppendingFormat:@"%@", [sender hostName]];
        [appDelegate setServerConnectionInformation:URLString];
        [appDelegate setServerHostName:hostNameString];
        
        // Upload local items when the central repository is resolved, not when found
        [appDelegate setServerFound:sender];
        [appDelegate tryUploadAllItems];
    }
}



- (void)netService:(NSNetService *)sender didUpdateTXTRecordData:(NSData *)data {
    NSLog(@"didUpdateTXTRecordData");
    
    NSString *dataString = [[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding];
    if (([sender hostName] != nil) &&
        ([sender name] != nil) &&
        ([sender port] != -1) &&
        (data != nil)) 
    {
        NSLog(@"-");
        NSLog(@"dataString:%@ from \"%@\"", dataString, [sender name]);
        NSLog(@"hostName:port = \"%@:%d\"", [sender hostName], [sender port]);
    }
}


- (void)netServiceDidPublish:(NSNetService *)sender {
    NSLog(@" netServiceDidPublish");
}


- (void)netServiceDidStop:(NSNetService *)sender {
    NSLog(@" netServiceDidStop");
    
}


- (void)netServiceWillPublish:(NSNetService *)sender {
    NSLog(@" netServiceWillPublish");
}


- (void)netServiceWillResolve:(NSNetService *)sender {
    NSLog(@" netServiceWillResolve");
}



#pragma mark User defined

- (NSString *)getMeaninglessString {
    return @"hello";
}


// Error handling code
- (void)handleError:(NSNumber *)error
{
    NSLog(@"An error occurred. Error code = %d", [error intValue]);
    
    // Handle error here
}

/*
 The 'domain' parameter passed to netServiceBrowser:didRemoveDomain:moreComing: and netServiceBrowser:didFindDomain:moreComing: may contain escaped characters. This function unescapes them before they are added to or removed from the list that is displayed to the user.
 */
- (NSString*) transmogrify:(NSString*)aString {
	
	NSString* tmp = [NSString stringWithString:aString];
	const char *ostr = [tmp UTF8String];
	const char *cstr = ostr;
	char *ptr = (char*) ostr;
	
	while (*cstr) {
		char c = *cstr++;
		if (c == '\\')
		{
			c = *cstr++;
			if (isdigit(cstr[-1]) && isdigit(cstr[0]) && isdigit(cstr[1]))
			{
				NSInteger v0 = cstr[-1] - '0';						// then interpret as three-digit decimal
				NSInteger v1 = cstr[ 0] - '0';
				NSInteger v2 = cstr[ 1] - '0';
				NSInteger val = v0 * 100 + v1 * 10 + v2;
				if (val <= 255) { c = (char)val; cstr += 2; }	// If valid three-digit decimal value, use it
			}
		}
		*ptr++ = c;
	}
	ptr--;
	*ptr = 0;
	return [NSString stringWithUTF8String:ostr];
}


- (void)startMonitoring {
    // Start browsing
    [self.netServiceBrowser searchForServicesOfType:@"_7dsbbs._tcp." inDomain:@""];
}


- (void)stopMonitoring {
    [self.netServiceBrowser stop];
}


@end
