//
//  Item.m
//  BBSApp
//
//  Created by Kiwoon Sung on 7/26/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <netinet/in.h>
#import <arpa/inet.h>

#import "BBSAppAppDelegate.h"

#import "Item.h"
#import "ItemXMLReader.h"

#import "BBSItemLocation.h"

@interface Item()
- (void)getSomethingFromTitle:(NSString *)wholeTitle;
@end


@implementation Item

@synthesize netService = _netService;
@synthesize fileName = _fileName;
@synthesize itemName = _itemName;
@synthesize shortDescription = _shortDescription;
@synthesize _URL;
@synthesize fields = _fields;

//@synthesize logger = m_logger;
@synthesize streamingParser = m_streamingParser;
@synthesize location;

@synthesize htmlString;

- (id)initWithNetService:(NSNetService *)netService fileName:(NSString *)fileName {
	if (self = [super init]) {
        self.location = BBSItemLocationPeer;
        
        self.fileName = fileName;
        self.netService = netService;
        
        NSLog(@"get the item information from the server");
        [self generateItemInstanceByXMLWithNetService:netService fileName:fileName];
        self.htmlString = [self generateHTMLString];
    }
    return self;
}


- (id)initWithFileNameInServer:(NSString *)fileName {
	if (self = [super init]) {
        self.location = BBSItemLocationServer;
        
        self.fileName = fileName;
        
        NSLog(@"get the item information from the server");
        [self generateItemInstanceByXMLFromServer:fileName];
        //[self printItem];
        self.htmlString = [self generateHTMLString];
        self.netService = nil;
    }
    return self;
}


- (id)initWithFileNameInLocal:(NSString *)fileName {
    if (self = [super init]) {
        self.location = BBSItemLocationLocal;
        
        self.fileName = fileName;
        
        NSLog(@"get the item information store in the local directory");
        NSLog(@"fileName: %@", fileName);
        [self generateItemInstanceByXMLFromLocal:fileName];
        self.htmlString = [self generateHTMLString];
        self.netService = nil;
    }
    return self;
}


//- (id)initWithFileNameInPeer:(NSString *)fileName (PeerInformation *):peerInformation {
//    return self;
//}

- (void)generateItemInstanceByXMLWithNetService:(NSNetService *)netService fileName:(NSString *)fileName {
    NSLog(@"generateItemInstanceByXMLWithNetService");
    
    NSError *parseError = nil;
    NSString *addressString = @"";
    
    // Retrieve the server information
    // Set up the cell.
    NSArray *address = [netService addresses];
    if ([address count] == 0) {
        NSLog(@"internal error: no ip address for host: %@", [netService hostName]);
    }
    
    if ([address count] == 1) {
        // Assume that this occurrs only when the devices such as iPhone/iPod Touch
        // are subject to connect to.
        NSLog(@"Seems like iPod Touch");

        struct sockaddr_in addrIn = *(struct sockaddr_in *)[[address objectAtIndex:0] bytes];
        addressString = [NSString stringWithFormat: @"%s", inet_ntoa(addrIn.sin_addr)];
        
        //addressString = [[NSString alloc]initWithData:[address objectAtIndex:0] encoding:NSUTF8StringEncoding];
        NSLog(@"addressString=%@", addressString);
    }
    
    NSString *URLString = @"";
    
    // Handle in a different way in case that iPod is about to be connected
    if ([addressString isEqualToString:@""]) {
        URLString = [@"http://" stringByAppendingString:[netService hostName]];
    } else {
        URLString = [@"http://" stringByAppendingString:addressString];
    }
    URLString = [URLString stringByAppendingString:@":"];
    URLString = [URLString stringByAppendingString:[@"" stringByAppendingFormat:@"%d", [netService port]]];
    URLString = [URLString stringByAppendingString:@"/"];
    URLString = [URLString stringByAppendingString:fileName];
    
    // Adding percent escapes
    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    NSLog(@"URL request after percent sequence adding: %@", URLString);
    
    self.streamingParser = [[ItemXMLReader alloc] init];
    [self.streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        NSLog(@"error code = %d", [parseError code]);
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"Item.m: NSXMLParserPrematureDocumentEndError: generateItemInstanceByXMLWithNetService");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
        
        // Don't let the self members be nil to avoid future memory violations
        self.itemName = @"Invalid Notice";
        self.fileName = fileName;
        self.shortDescription = @"<b>The contents of this notice seem to be broken or not a valid format. Or, chances are that the server is not working at this moment.</b> This notice is stored at the central repository so the administrator or the own of this file should remove this item from the repository.";
        
        self.fields = [[NSMutableArray alloc]initWithCapacity:1];
        [self.fields addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:
                                @"administrator@....com", @"Contact",
                                nil]];
    } else {    
        NSLog(@"item object for this view has been obtained from the server");
        
        // Only assign the information acquired from the XML file, not the object itself.
        self.itemName = self.streamingParser.itemName;
        self.fileName = self.streamingParser.fileName;
        self.shortDescription = self.streamingParser.shortDescription;
        self.fields = self.streamingParser.fields;
    }
    NSLog(@"itemName=%@", self.itemName);
}


- (void)generateItemInstanceByXMLFromServer:(NSString *)fileName {
    NSLog(@"generateItemInstanceByXMLFromServer");
    
    NSError *parseError = nil;
    
    // Retrieve the server information
    // Set up the cell.
	BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    //NSString *serverInfo = [appDelegate getServerConnectionInformation];
    
//    NSString *URLString = [@"http://" stringByAppendingString:[appDelegate getServerConnectionInformation]];
    NSString *URLString = [@"" stringByAppendingString:[appDelegate getServerConnectionInformation]];
//    URLString = [URLString stringByAppendingString:@":"];
//    URLString = [URLString stringByAppendingString:serverListeningPort];
//    URLString = [URLString stringByAppendingString:@"/"];
    URLString = [URLString stringByAppendingString:fileName];
    
   
    NSLog(@"URL request: %@", URLString);
    
    // If escape sequence in the URL
    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    NSLog(@"URL request after percent sequence adding: %@", URLString);
    
    self.streamingParser = [[ItemXMLReader alloc] init];
    [self.streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        NSLog(@"error code = %d", [parseError code]);
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"Item.m: NSXMLParserPrematureDocumentEndError: generateItemInstanceByXMLFromServer");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }

        // Don't let the self members be nil to avoid future memory violations
        self.itemName = @"Invalid Notice";
        self.fileName = fileName;
        self.shortDescription = @"<b>The contents of this notice seem to be broken or not a valid format. Or, chances are that the server is not working at this moment.</b> This notice is stored at the central repository so the administrator or the own of this file should remove this item from the repository.";
        
        self.fields = [[NSMutableArray alloc]initWithCapacity:1];
        [self.fields addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:
                                @"administrator@....com", @"Contact",
                                nil]];
    } else {    
        NSLog(@"item object for this view has been obtained from the server");
        
        // Only assign the information acquired from the XML file, not the object itself.
        self.itemName = self.streamingParser.itemName;
        self.fileName = self.streamingParser.fileName;
        self.shortDescription = self.streamingParser.shortDescription;
        self.fields = self.streamingParser.fields;
    }
    
    
    NSLog(@"itemName=%@", self.itemName);

    // Don't free the following members here
    //[parseError release];
//    [self.streamingParser release];
}


- (void)generateItemInstanceByXMLFromLocal:(NSString *)fileName {
    NSLog(@"generateItemInstanceByXMLFromLocal: fileName=%@", fileName);
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSData *xmlData = nil;
    NSError *parseError = nil;

    fileName = [@"/" stringByAppendingString:fileName];
    NSString *fullPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:fileName];
    NSLog(@"fullPath of XML file:%@", fullPath);
    if (![fileManager fileExistsAtPath:fullPath]) {
        NSLog(@"file not found: %@", fullPath);
    }
    else {
        xmlData = [[NSData alloc]initWithContentsOfFile:fullPath];
        NSLog(@"file: %@: found and assigned to xmlData object", fullPath);
    }
    if (xmlData == nil) {
        NSLog(@"xml source data has not beed loaded properly");
    }
    else {
        self.streamingParser = [[ItemXMLReader alloc] init];
        [self.streamingParser parseXMLWithData:xmlData parseError:&parseError];
        
        if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
            NSLog(@"error code = %d", [parseError code]);
            switch([parseError code]) {
                case NSXMLParserPrematureDocumentEndError:
                    NSLog(@"Item.m: NSXMLParserPrematureDocumentEndError: generateItemInstanceByXMLFromLocal");
                    // See the end of this source to check error types.
                    // The corresponding actions should be addressed here soon.
                    break;
                default:
                    NSLog(@"error code = %d", [parseError code]);
            }
        }
        else {
            NSLog(@"item object for this view has been obtained from the server");
            
            // Only assign the information acquired from the XML file, not the object itself.
            self.itemName = self.streamingParser.itemName;
            self.fileName = self.streamingParser.fileName;
            self.shortDescription = self.streamingParser.shortDescription;
            self.fields = self.streamingParser.fields;
            
        }
    }

    NSLog(@"itemName=%@", self.itemName);
    //
//    [fileManager release];
//    [xmlData release];
//    [parseError release];
    
    // MEMO: If you release the following variable, ItemViewController called by LocalItemTableViewController crashes in a very weird way. It is okay when you set the location flag as BBSItemLocationServer. Only when BBSItemLocationLocal is set, the crash occurrs. The reason has not been revealed yet. Why is that?
//    [fullPath release];
}



- (void)printItem {
    NSLog(@"---------");
    NSLog(@"printItem");
    NSLog(@"itemName: %@", self.itemName);
    NSLog(@"fileName: %@", self.fileName);
    NSLog(@"shortDescription: %@", self.shortDescription);
    NSLog(@"URL: %@", [NSString stringWithContentsOfURL:self._URL]);
    
    NSLog(@"fields count: %d", [self.fields count]);
    
    //    for (NSMutableArray *temp in self.itemObject.fields) {
    NSUInteger count = [self.fields count];
    NSUInteger index = 0;
    //    NSDictionary *myDictionary;
    for (index = 0; index < count; index++) {
        NSDictionary *oneDictionary = [self.fields objectAtIndex:index];
        //        NSString *fieldName = [[self.itemObject.fields objectAtIndex:index] objectForKey:@"field"];
        //        NSString *value = [[self.itemObject.fields objectAtIndex:index] objectForKey:@"data"];
        NSEnumerator *enumerator = [oneDictionary keyEnumerator];
        NSString *key;
        
        while ((key = [enumerator nextObject])) {
            NSString *valueString = [oneDictionary objectForKey:key];
            valueString = [valueString stringByAppendingString:@"END"];
            NSLog(@"index:%d fieldName:%@ value:%@", index, key, valueString);
            NSLog(@"valueString length: %u", [valueString length]);
        }
        
    }
}


// Call this only after initializing this instance and generateItemInstanceByXML
- (NSString *)generateHTMLString {
    NSString *html = @"";
    NSString *bodyHTML = @"";
    NSString *headerHTML = @"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\"><html><meta name=\"viewport\" content=\"width=400\"/><body><font face=\"Arial\">";
    NSString *footerHTML = @"</font></body></html>";
    BBSAppAppDelegate *appDelegate = [[UIApplication sharedApplication]delegate];
    NSString *selfURLString = [appDelegate getSelfConnectionInformation];
    NSLog(@"selfURLString = %@", selfURLString);
    
    // Generate the body HTML strings according to the contents in the Item instance
    if ((self.itemName == nil) || (self.shortDescription == nil)) {
        // These two members are mandatory
        NSLog(@"ERROR: generateHTMLString");
        bodyHTML = @"<h3>The contents of this item is invalid.</h3>";
    } else {
        // itemName
        bodyHTML = [bodyHTML stringByAppendingString:@"<h2>"];
        bodyHTML = [bodyHTML stringByAppendingString:self.itemName];
        bodyHTML = [bodyHTML stringByAppendingString:@"</h2><h3>"];
        
        // shortDescription
        // this is a mandatory field but another description tag might be there.
        //bodyHTML = [bodyHTML stringByAppendingString:@"Brief"];
        bodyHTML = [bodyHTML stringByAppendingString:@"</h3><p>"];
        bodyHTML = [bodyHTML stringByAppendingString:self.shortDescription];
        bodyHTML = [bodyHTML stringByAppendingString:@"<p>"];
        
        
        // For each user-defined field except the tag, "ImageLink"
        NSUInteger count = [self.fields count];
        NSUInteger index = 0;
        NSString *key = @"";
        NSEnumerator *enumerator;
        NSDictionary *oneDictionary;

        for (index = 0; index < count; index++) {
            oneDictionary = [self.fields objectAtIndex:index];
            enumerator = [oneDictionary keyEnumerator];
                        
            while ((key = [enumerator nextObject])) {
                NSString *valueString = [oneDictionary objectForKey:key];
                key = [key stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                valueString = [valueString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                NSLog(@"index:%d fieldName:%@ value:%@", index, key, valueString);
                NSLog(@"valueString length: %u", [valueString length]);
                
                // "ImageFileName" is an optional tag. Just for demonstrating
                // Should be indicating an image in this version.
                if ([key hasPrefix:@"ImageFileName"]) {
                    bodyHTML = [bodyHTML stringByAppendingString:@"<h3>"];
                    bodyHTML = [bodyHTML stringByAppendingString:@"Image (trial field)</h3>"];
                    bodyHTML = [bodyHTML stringByAppendingString:@"<p>only available when you connect to the original publisher's device/host"];
                    bodyHTML = [bodyHTML stringByAppendingString:@"<p><img src=\""];
                    
                    BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
                    switch (self.location) {
                        case BBSItemLocationPeer:
                            bodyHTML = [bodyHTML stringByAppendingFormat:@"http://%@:%d/", [self.netService hostName], [self.netService port]];
                            bodyHTML = [bodyHTML stringByAppendingString:valueString];
                            bodyHTML = [bodyHTML stringByAppendingString:@"\">"];
                            break;
                        case BBSItemLocationServer:
                            bodyHTML = [bodyHTML stringByAppendingString:selfURLString];
                            bodyHTML = [bodyHTML stringByAppendingString:valueString];
                            bodyHTML = [bodyHTML stringByAppendingString:@"\">"];
                            break;
                        case BBSItemLocationLocal:
                            bodyHTML = [bodyHTML stringByAppendingString:[appDelegate getSelfConnectionInformation]];
                            bodyHTML = [bodyHTML stringByAppendingString:valueString];
                            bodyHTML = [bodyHTML stringByAppendingString:@"\">"];
                            break;
                        default:
                            break;
                    }
                } else { 
                    // All other fields are user-defined ones
                    // field name
                    bodyHTML = [bodyHTML stringByAppendingString:@"<h3>"];
                    bodyHTML = [bodyHTML stringByAppendingString:key];
                    bodyHTML = [bodyHTML stringByAppendingString:@"</h3><p>"];
                    
                    // value
                    bodyHTML = [bodyHTML stringByAppendingString:valueString];
                    bodyHTML = [bodyHTML stringByAppendingString:@"<p>"];
                }
            }
            
        }
        
            
    }
    
    
    
    // Combining all strings together
    html = [html stringByAppendingString:headerHTML];
    html = [html stringByAppendingString:bodyHTML];
    html = [html stringByAppendingString:footerHTML];
    
    NSLog(@"generated HTML: %@", html);
    
    return html;
}


- (void)getSomethingFromTitle:(NSString *)wholeTitle {
    
}

- (void) dealloc
{
    [self.fields release];
//    [self.logger release];
    [self.streamingParser release];
    [self.fileName release];
    [self.itemName release];
    [self.shortDescription release];
    [self._URL release];
    [self.fields release];
    [self.htmlString release];
    
    [super dealloc];
}


- (NSString *)getItemName {
    return self.itemName;
}


- (NSString *)getShortDescription {
    return self.shortDescription;
}


- (NSMutableArray *)getFields {
    return self.fields;
}


- (NSURL *)getURL {
    return self._URL;
}


@end
