//
//  ItemInformation.h
//  BBSApp
//
//  Created by Kiwoon Sung on 7/27/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <UIKit/UIKit.h>


// This is the class describing the item information.
// It is differ from Item class since this one describes which BBS items the server contains in the central repository, whereas Item class describes the BBS item itself in detail. 
// When it comes to the item information from the peer, that information can be
// acquired from the broadcasted information by ZeroConf.
// This class only contains the data defined as the item list information XML.
// Therefore, three mendatory fields exist; fileName, itemName, and shortDescription. These are not the tags but fields. Each entry tag in the following example has fileName, itemName, and shortDescription together. 
// This XML file is the one that you can download from the central repository when you request the information about which BBS items are stored in the central repository.
//
// For example,
//<?xml version="1.0" encoding="utf-8"?>
//<bbs name = "ItemListFromServer">
//<entry>
//<fileName>iPhoneSale.xml</fileName>
//<itemName>iPhone3G For Sale</itemName>
//<shortDescription>Brand New 3G 16GB Black iPhone With Receipt</shortDescription>
//</entry>
//
//
//<entry>
//<fileName>ForSale.xml</fileName>
//<itemName>Car For Sale Now</itemName>
//<shortDescription>This is the short description of the car for sale. Should be very long and you need to see what happens. How are you gonna cover the long string in the text view control? Apple..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..Very long string..This is no longer a short description then.</shortDescription>
//</entry>
//
//
//</bbs>

typedef enum {
    kItemSharingPublic = 1,
    kItemSharingPrivate = 2,
    kItemSharingDraft = 3, // this is the default. draft items are stored in home directory, not in a sub-folder as if other types are stored there.
    kItemSharingRFU = 4, // reserved for future-use and adding a type of sharing status.
} ItemInformationSharingStatus;



@interface ItemInformation : NSObject {
    NSString *_fileName;  // need to see if this filename contains "@" character.
    NSString *_itemName;
    NSString *_shortDescription;
    
    // Newly added: ver 1.3
    ItemInformationSharingStatus sharingStatus;
    
    NSString *_itemUUID;
    NSString *_lastModified;
    NSString *_authorID;
    NSNetService *_netService;
    NSString *_tagString; // a single string with zero or more tag(s)
    
    
    // Newly added: hostname
    // For the compatability with the CLI version, just add this member
    // and see if it's nil or not when it is being diplayed.
    // If nil, assign "Unknown" or a word describing the item is locally stored
    // should be used.
//    NSString *_hostname; // implement this later
}

@property (nonatomic, retain) NSString *fileName;
@property (nonatomic, retain) NSString *itemName;
@property (nonatomic, retain) NSString *shortDescription;
@property ItemInformationSharingStatus sharingStatus;
@property (nonatomic, retain) NSString *itemUUID;
@property (nonatomic, retain) NSString *lastModified;
@property (nonatomic, retain) NSString *authorID;
@property (nonatomic, retain) NSNetService *netService;
@property (nonatomic, retain) NSString *tagString;

//@property (nonatomic, retain) NSString *hostname;

@end
