//
//  ItemTableViewCell.m
//  BBSApp
//
//  Created by Kiwoon Sung on 7/27/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import "Contants.h"

#import "ItemInformation.h"
#import "ItemInformationTableViewCell.h"


@interface ItemInformationTableViewCell()
- (UILabel *)newLabelWithPrimaryColor:(UIColor *)primaryColor selectedColor:(UIColor *)selectedColor fontSize:(CGFloat)fontSize bold:(BOOL)bold;
@end

@implementation ItemInformationTableViewCell


@synthesize itemName;
@synthesize shortDescription;
@synthesize itemNameLabel = _itemNameLabel;
@synthesize shortDescriptionLabel = _shortDescriptionLabel;

- (id)initWithFrame:(CGRect)frame reuseIdentifier:(NSString *)reuseIdentifier {
	if (self = [super initWithFrame:frame reuseIdentifier:reuseIdentifier]) {
		// Initialization code
        
        UIView *myContentView = self.contentView;
        
        // A label that displays the item name of the item.
        self.itemNameLabel = [self newLabelWithPrimaryColor:[UIColor blackColor] selectedColor:[UIColor whiteColor] fontSize:ITEM_NAME_FONT_SIZE bold:YES];
        self.itemNameLabel.textAlignment = UITextAlignmentLeft;
        self.itemNameLabel.backgroundColor = [UIColor clearColor];
        self.itemNameLabel.opaque = YES;
        [myContentView addSubview:self.itemNameLabel];
        [self.itemNameLabel release];
        
        // A label that displays the short description of the item
        // This information is only available on the item table view,
        // not in the item view
        self.shortDescriptionLabel = [self newLabelWithPrimaryColor:[UIColor grayColor] selectedColor:[UIColor lightGrayColor] fontSize:SHORT_DESCRIPTION_FONT_SIZE bold:NO];
        self.shortDescriptionLabel.textAlignment = UITextAlignmentLeft;
        self.shortDescriptionLabel.backgroundColor = [UIColor clearColor];
        [myContentView addSubview:self.shortDescriptionLabel];
        [self.shortDescriptionLabel release];
        
	}
	return self;
}


- (ItemInformation *)getItemInformation {
    return _itemInformation; 
}

- (void)setItemInformation:(ItemInformation *)newItemInformation {
//    NSLog(@"setItemInformation called:");
    [newItemInformation retain];
    [_itemInformation release];
    _itemInformation = newItemInformation;
    
    NSLog(@"itemName = %@", newItemInformation.itemName);
    self.itemNameLabel.text = newItemInformation.itemName;
    NSLog(@"shortDescription = %@", newItemInformation.shortDescription);
    self.shortDescriptionLabel.text = newItemInformation.shortDescription;
    
    [self setNeedsDisplay];
}

- (void)layoutSubviews {

    [super layoutSubviews];
    CGRect contentRect = self.contentView.bounds;
    
    // In this example we will never be editing, but this illustrates the appropriate pattern
    if (!self.editing) {
        
        CGFloat boundsX = contentRect.origin.x;
        CGRect frame;
        
        // Place the location label.
        frame = CGRectMake(boundsX + LEFT_COLUMN_OFFSET, UPPER_ROW_TOP, LEFT_COLUMN_WIDTH, 20);
        self.itemNameLabel.frame = frame;
        
        // Place the date label.
        frame = CGRectMake(boundsX + LEFT_COLUMN_OFFSET, LOWER_ROW_TOP, LEFT_COLUMN_WIDTH, 18);
        self.shortDescriptionLabel.frame = frame;     
    }
}

        
- (void)setSelected:(BOOL)selected animated:(BOOL)animated {

	[super setSelected:selected animated:animated];

	// Configure the view for the selected state
    UIColor *backgroundColor = nil;
    if (selected) {
        backgroundColor = [UIColor clearColor];
    } else {
        backgroundColor = [UIColor clearColor];
    }
    
    self.itemNameLabel.backgroundColor = backgroundColor;
    self.itemNameLabel.highlighted = selected;
    self.itemNameLabel.opaque = !selected;
    
    self.shortDescriptionLabel.backgroundColor = backgroundColor;
    self.shortDescriptionLabel.highlighted = selected;
    self.shortDescriptionLabel.opaque = !selected;            
}


- (void)dealloc {
    [itemName release];
    [shortDescription release];
    [self.itemNameLabel release];
    [self.shortDescriptionLabel release];
    
	[super dealloc];
}

- (UILabel *)newLabelWithPrimaryColor:(UIColor *)primaryColor selectedColor:(UIColor *)selectedColor fontSize:(CGFloat)fontSize bold:(BOOL)bold
{
	/*
     Create and configure a label.
     */
    
    UIFont *font;
    if (bold) {
        font = [UIFont boldSystemFontOfSize:fontSize];
    } else {
        font = [UIFont systemFontOfSize:fontSize];
    }
    
    /*
     Views are drawn most efficiently when they are opaque and do not have a clear background, so set these defaults.  To show selection properly, however, the views need to be transparent (so that the selection color shows through).  This is handled in setSelected:animated:.
     */
	UILabel *newLabel = [[UILabel alloc] initWithFrame:CGRectZero];
	newLabel.backgroundColor = [UIColor clearColor];
	newLabel.opaque = YES;
	newLabel.textColor = primaryColor;
	newLabel.highlightedTextColor = selectedColor;
	newLabel.font = font;
	
	return newLabel;
}

@end
