//
//  ItemViewController.m
//  BBSApp
//
//  Created by Kiwoon Sung on 7/26/08.
//  Copyright 2008 Columbia University. All rights reserved.
//


#import "Contants.h"
#import "SLogger.h"
#import "BBSItemLocation.h"

#import "BBSAppAppDelegate.h"
#import "ItemViewController.h"
#import "ItemXMLReader.h"


@implementation ItemViewController

@synthesize myTableView;  // No more in use due to the scroll view delegate
@synthesize logger = m_logger;
@synthesize itemObject = m_itemObject;
@synthesize containerView = m_containerView;
@synthesize fileName = m_fileName;
@synthesize itemView;
@synthesize activityIndicatorView;
@synthesize givenSectionTitle;
@synthesize downloadFrom;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        self.logger = [[SLogger alloc]initWithClassName:@"ItemViewController" isDebugMode:NO];
        
        [self.logger called:@"initWithNibName"];
		// Initialization code
        self.title = @"Notices";
        self.navigationItem.title = @"Notices";
        self.givenSectionTitle = @"Notices in ";
        
        // Download the corresponding XML contents from the server
        
        //self.itemObject = [tempItem generateItemInstanceByXML:@"TEST"];
	}
	return self;
}


- (id)initWithItemFileName:(NSString *)fileName itemLocation:(BBSItemLocation)itemLocation{
	if (self = [super initWithNibName:nil bundle:nil]) {
        self.logger = [[SLogger alloc]initWithClassName:@"ItemViewController" isDebugMode:YES];
        
        //        [self.logger called:@"initWithItemFileName"];
		// Initialization code
        self.title = @"Notice";
        self.navigationItem.title = @"Notice";
        self.fileName = fileName;
        
        self.downloadFrom = itemLocation;
        self.givenSectionTitle = @"Notices from";
        // Initialize the item instance for this view
        [self loadItem:self.fileName];
        
        //self.downloadFrom = BBSItemLocationServer;
        
        // The following code should be removed with the code calling hideActivityIndicator together
        // Otherwise, hideActivityIndicator will try to free an non-allocated memory
        //[self showActivityIndicator];
        
        // Download the corresponding XML contents from the server
        
        // Initialize the item instance for this view
        //Item *tempItem = [[Item alloc] init;
        //self.itemObject = [tempItem generateItemInstanceByXML:@"TEST"];
	}
	return self;
    
    
}


- (id)initWithNetService:(NSNetService *)netService fileName:(NSString *)fileName{
	if (self = [super initWithNibName:nil bundle:nil]) {
        self.title = [netService name];
        self.navigationItem.title = [netService name];
        self.fileName = fileName;
        self.givenSectionTitle = [@"Notices in " stringByAppendingString:[netService name]];
        
        [self loadItemWithNetService:netService fileName:self.fileName];
        
	}
	return self;
    
    
}


- (void)loadItemWithNetService:(NSNetService *)netService fileName:(NSString *)fileName {
    NSLog(@"loadItem called in ItemViewController");
    
    self.itemObject = [[Item alloc]initWithNetService:netService fileName:fileName];
}



- (void)loadItem:(NSString *)fileName {
    NSLog(@"loadItem called in ItemViewController");
    
    // Here is the point where the location that the item should be downloaded from is decided.
    switch (self.downloadFrom) {
        case BBSItemLocationServer:
            // Assign the downloaded object that is acqured by extracting the xml file to the internal object for the item
            self.itemObject = [[Item alloc]initWithFileNameInServer:fileName];
            break;
        case BBSItemLocationLocal:
            self.itemObject = [[Item alloc]initWithFileNameInLocal:fileName];
            break;
        case BBSItemLocationPeer:
            //            self.itemObject = [[Item alloc]initWithFileNameInPeer:fileName peerInformation:peerInformation];
            break;
        default:
            break;
    }
    
}



// Implement loadView if you want to create a view hierarchy programmatically
- (void)loadView {
    NSLog(@"loadView: ItemViewController");
    
    NSLog(@"loadItem finished");
    CGRect viewRectFull = [[UIScreen mainScreen] applicationFrame];
    viewRectFull.size.height = 0;
    UIWebView *localWebView = [[UIWebView alloc] initWithFrame:viewRectFull];
    localWebView.backgroundColor = [UIColor colorWithWhite:50.0 alpha:20.0];

    localWebView.delegate = self;
    localWebView.scalesPageToFit = YES;
    
    NSURL *_sURL = [[NSURL alloc]initWithString:@"http:"];
    NSString *html = [self.itemObject htmlString];
    //NSString *html = @"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\"><html><meta name=\"viewport\" content=\"width=320\"/><body><font face=\"Arial\"><h2>iPhone Item Name</h2><br><h3>Description</h3><p>Introducing iPhone 3G. With fast 3G wireless technology, GPS mapping, support for enterprise features like Microsoft Exchange, and the new App Store, iPhone 3G puts even more features at your fingertips. And like the original iPhone, it combines three products in one — a revolutionary phone, a widescreen iPod, and a breakthrough Internet device with rich HTML email and a desktop-class web browser. iPhone 3G. It redefines what a mobile phone can do- again.<p><h3>Price</h3><p>$199.00<p><h3>Field Name1</h3><p>Value as a string<p><h3>Field Name2</h3><p>Value as a string<p><h3>Field Name3</h3><p>Value as a string<p><h3>Image</h3><p><img src=\"http://192.168.1.4/iphone.jpg\"></font></body></html>";

    [localWebView loadHTMLString:html baseURL:_sURL];

    self.view = localWebView;
    [localWebView release];
   

    
    /* try to use web view instead of the following
     CGSize preferredSubViewSize = CGSizeMake(256, 200);
     
    // Create and store a container view
    CGRect viewRectFull = [[UIScreen mainScreen] applicationFrame];
    UIScrollView *localContainerView = [[UIScrollView alloc] initWithFrame:viewRectFull];
    localContainerView.pagingEnabled = NO;
    localContainerView.contentSize = CGSizeMake(localContainerView.frame.size.width, localContainerView.frame.size.height *1);
    localContainerView.showsVerticalScrollIndicator = YES;
    localContainerView.scrollsToTop = NO;
    localContainerView.delegate = self;
    localContainerView.autoresizesSubviews = YES;
    
    self.containerView = localContainerView;
    [localContainerView release];
    
    self.containerView.backgroundColor = [UIColor whiteColor];
    
    CGRect viewRect = CGRectMake(0,0,
        preferredSubViewSize.width,
        preferredSubViewSize.height);
    
    ItemView *localItemView = [[ItemView alloc] initWithFrameAndItem:viewRect itemObject:self.itemObject];
    self.itemView = localItemView;
    [localItemView release];
        
    // Add the about view to the containerView
    [self.containerView addSubview:self.itemView];
    // [containerView addSubview:myTextView];
    
    self.itemView.viewController = self;
    self.view = self.containerView;*/
    
}

/*
 If you need to do additional setup after loading the view, override viewDidLoad.
- (void)viewDidLoad {
}
 */


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
    NSLog(@"didReceiveMemoryWarning");
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [myTableView release];
    [self.containerView release];
    [self.logger release];
    [self.itemObject release];
    [self.logger release];
    
	[super dealloc];
}

#pragma mark = UITableView delegates

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellEditingStyleNone;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 1;
}

- (UITableViewCell *)obtainTableCellForRow:(NSInteger)row
{
	UITableViewCell *cell = nil;
    
	
	return cell;
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{

	return @"TempTitle";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return 2;
}


- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
	CGFloat result;
	
	switch ([indexPath row])
	{
		case 0:
		{
			result = kUIRowHeight;
			break;
		}
		case 1:
		{
			result = kUIRowLabelHeight;
			break;
		}
	}
    
	return result;
}



// utility routine leveraged by 'cellForRowAtIndexPath' to determine which UITableViewCell to be used on a given row
//
//- (UITableViewCell *)obtainTableCellForRow:(NSInteger)row
//{
//	UITableViewCell *cell = nil;
//    
//	if (row == 0)
//		cell = [myTableView dequeueReusableCellWithIdentifier:kDisplayCell_ID];
//	else if (row == 1)
//		cell = [myTableView dequeueReusableCellWithIdentifier:kSourceCell_ID];
//	
//	if (cell == nil)
//	{
//		if (row == 0)
//			cell = [[[DisplayCell alloc] initWithFrame:CGRectZero reuseIdentifier:kDisplayCell_ID] autorelease];
//		else if (row == 1)
//			cell = [[[SourceCell alloc] initWithFrame:CGRectZero reuseIdentifier:kSourceCell_ID] autorelease];
//	}
//	
//	return cell;
//}


// to determine which UITableViewCell to be used on a given row.
//
- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	NSInteger row = [indexPath row];
	
	UITableViewCell *cell = [self obtainTableCellForRow:row];
	
	
	return cell;
}


#pragma mark UIScrollViewDelegate

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    NSLog(@"ItemViewController: scrollViewDidEndDecelerating");
}


- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    NSLog(@"ItemViewController: scrollViewDidScroll");
    
    
}


#pragma mark UIWebViewDelegate

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
    NSLog(@"shouldStartLoadWithRequest");
    return YES;
}


- (void)webViewDidStartLoad:(UIWebView *)webView {
        NSLog(@"webViewDidStartLoad");

}


- (void)webViewDidFinishLoad:(UIWebView *)webView {
    NSLog(@"webViewDidFinishLoad");
    
    [self hideActivityIndicator];
}


- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error {
    NSLog(@"didFailLoadWithError");

    [self hideActivityIndicator];
}


- (void)loadWebViewContentsTest:(UIWebView *)tempWebView {
    // Request
    NSURL *_URL = [[NSURL alloc]initWithString:@"http:"];
    
    NSString *html = @"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\"><html><meta name=\"viewport\" content=\"width=320\"/><body><font face=\"Arial\"><h2>iPhone Item Name</h2><br><h3>Description</h3><p>Introducing iPhone 3G. With fast 3G wireless technology, GPS mapping, support for enterprise features like Microsoft Exchange, and the new App Store, iPhone 3G puts even more features at your fingertips. And like the original iPhone, it combines three products in one — a revolutionary phone, a widescreen iPod, and a breakthrough Internet device with rich HTML email and a desktop-class web browser. iPhone 3G. It redefines what a mobile phone can do- again.<p><h3>Price</h3><p>$199.00<p><h3>Field Name1</h3><p>Value as a string<p><h3>Field Name2</h3><p>Value as a string<p><h3>Field Name3</h3><p>Value as a string<p><h3>Image</h3><p><img src=\"http://192.168.1.4/iphone.jpg\"></font></body></html>";
    //NSURLRequest *requestURL = [[NSURLRequest alloc] initWithURL:_URL];
    
    
    [tempWebView loadHTMLString:html baseURL:_URL];
}    


- (void) showActivityIndicator {
    // Show the activity indicator before parsing the contents.
    UIActivityIndicatorView *localActivityIndicatorView;
    CGRect frame = CGRectMake(kActivityIndicatorXOrigin, kActivityIndicatorYOrigin, kActivityIndicatorSize, kActivityIndicatorSize);
    localActivityIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:frame];
    [localActivityIndicatorView startAnimating];
    localActivityIndicatorView.hidesWhenStopped = YES;
    localActivityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleGray;
    [localActivityIndicatorView sizeToFit];
    localActivityIndicatorView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
                                                   UIViewAutoresizingFlexibleRightMargin |
                                                   UIViewAutoresizingFlexibleTopMargin |
                                                   UIViewAutoresizingFlexibleBottomMargin);
    
    self.activityIndicatorView = localActivityIndicatorView;
    
    [self.view addSubview:self.activityIndicatorView];
    
}


- (void)hideActivityIndicator {
    [self.activityIndicatorView stopAnimating];
    
}


- (void)refreshAction:(id)sender {
    NSLog(@"refresh button clicked");
    [self displayStatusOnToolbar:@"Refreshing" showIndicator:YES showStopButton:NO showRefreshButton:NO];
    

}


- (void)stopAction:(id)sender {
    NSLog(@"stop button clicked");
    
    [self displayStatusOnToolbar:@"Stopped" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}



- (void)displayStatusOnToolbar:(NSString *)text 
                 showIndicator:(BOOL)showIndicator
                showStopButton:(BOOL)showStopButton
             showRefreshButton:(BOOL)showRefreshButton 
{
    CGRect infoRect = CGRectMake(0.0, 0.0, 300, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(0.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = text; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// Create and add an indicator if needed
	NSMutableArray *items = [[NSMutableArray alloc]init];
    if (showIndicator) {
        // Create an activity indicator at the rightmost of the toolbar
        UIActivityIndicatorView *activityIndicator;
        CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
        activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
        activityIndicator.opaque = YES;
        activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
        [activityIndicator startAnimating];
        
        UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
        
        [items addObject:indicatorItem];
        
        [activityIndicator release];
    }
    
    // Mandatory field: add systemItem
    [items addObject:systemItem];
    [items addObject:flexItem];
    
    // Create and add a stop button
    if (showStopButton) {
        // Create a stop tool bar button
        UIBarButtonItem *stopItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop 
                                                                                  target:self 
                                                                                  action:@selector(stopAction:)];
        
        [items addObject:stopItem];
        [stopItem release];
    }
    
    // Create and add a refresh button if needed
    if (showRefreshButton) {
        // Create a refresh tool bar button for future use
        UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                     target:self 
                                                                                     action:@selector(refreshAction:)];
        
        [items addObject:refreshItem];
        [refreshItem release];
    }
    
    // Construct the tool bar
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
}



@end
