
//
//  ItemXMLReader.h
//  BBSApp
//
//  Created by Kiwoon Sung on 7/27/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "SLogger.h"

/*
    This class uses NSXMLParser to map the contents of an XML document to Objective-C model objects.
   
    The following is the sample that is subject to be parsed by this wrapper class.
    
    There are two specific fields (not tags) that should be placed in the XML representing BBS item.
    <field>itemName</field>
    <field>shortDescription</field>
 
    The above two fields are enclosed in <entry> tag separately. 
    On the server side, the central repository program parses the BBS item files (xml files),
    looks for those fields, composing the item list and reply the information when the client requests
    item list in the server. The server also needs to add the file name information to compose the file.
 
    This information is for the client to get the item list information and use it to compose
    item list menu.
    

 <?xml version="1.0" encoding="utf-8"?>
 <bbs name = "iPhoneSale.xml">
 
 <entry>
 <field>itemName</field> 
 <data>iPhone3G For Sale</data>
 </entry>
 <entry>
 <field>shortDescription</field>
 <data>Brand New 3G 16GB Black iPhone With Receipt</data>
 </entry>
 
 
 <entry>
 <field>Product</field>
 <data>iPhone3G</data>
 </entry>
 <entry>
 <field>Price</field>
 <data>$550.00</data>
 </entry>
 <entry>
 <field>Description</field>
 <data>Brand new 3G 16GB Black iPhone was only opened when purchased/ includes receipt for warranty purposes. Will Fedex overnight, for security issues will only accept PayPal. OR, I am already around you if you find me in the peer item list so we can talk. I bring this thing everyday. Just talk to me over the messenger such as (...click here...). Email with any other questions if you are not around.</data>
 </entry>
 <entry>
 <field>Comments</field>
 <data>This sample was picked up from craigslist.org for the purpose of testing only</data>
 </entry>
 <entry>
 <field>ReplyTo</field>
 <data>sale-771073040@craigslist.org</data>
 </entry>
 <entry>
 <field>Date</field>
 <data>2008-07-26, 5:25PM EDT</data>
 </entry>
 <entry>
 <field>Location</field>
 <data>Auburn</data>
 </entry>
 <entry>
 <field>Image</field>
 <data>http://localhost/iphone.png</data>
 </entry>
 </bbs>
 
*/
@class Item;

@interface ItemXMLReader: NSObject {

       
@private 
    //Item *_currentItemObject;
    NSMutableString *_currentItemFieldName;
    NSMutableString *_contentOfCurrentItemProperty;
    
    NSMutableString *m_itemName;
    NSMutableString *m_fileName;
    NSMutableString *m_shortDescription;
    NSURL *m_URL;
    NSMutableArray *m_fields;
    
    SLogger *m_logger;
}

//@property (nonatomic, retain) Item *currentItemObject;
@property (nonatomic, retain) NSMutableString *currentItemFieldName;
@property (nonatomic, retain) NSMutableString *contentOfCurrentItemProperty;
@property (nonatomic, retain) SLogger *logger;

@property (nonatomic, retain) NSMutableString *itemName;
@property (nonatomic, retain) NSMutableString *fileName;
@property (nonatomic, retain) NSMutableString *shortDescription;
@property (nonatomic, retain) NSURL *URL;
@property (nonatomic, retain) NSMutableArray *fields;


- (void)parseXMLFileAtURL:(NSURL *)_URL parseError:(NSError **)error;
- (void)parseXMLWithData:(NSData *)xmlData parseError:(NSError **)error;



@end
