//
//  ItemXMLReader.m
//  BBSApp
//
//  Created by Kiwoon Sung on 7/27/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import "Contants.h"

#import "ItemXMLReader.h"

@class Item;

static NSUInteger parsedItemCounter;

@implementation ItemXMLReader

//@synthesize currentItemObject = _currentItemObject;
@synthesize currentItemFieldName = _currentItemFieldName;
@synthesize contentOfCurrentItemProperty = _contentOfCurrentItemProperty;
@synthesize logger = m_logger;

@synthesize itemName = m_itemName;
@synthesize fileName = m_fileName;
@synthesize shortDescription = m_shortDescription;
@synthesize URL = m_URL;
@synthesize fields = m_fields;

- (id) init {
    self = [super init];
    if (self != nil) {
//        self.currentItemObject = [[Item alloc]init];
        self.fields = [[NSMutableArray alloc]init];
        self.logger = [[SLogger alloc]initWithClassName:@"ItemXMLReader" isDebugMode:YES];
    }
    return self;
}


- (void)dealloc {
//    [self.currentItemObject release];
    [self.contentOfCurrentItemProperty release];
    [self.currentItemFieldName release];
    [self.logger release];
    [self.itemName release];
    [self.fileName release];
    [self.shortDescription release];
    [self.URL release];
    [self.fields release];
    
	[super dealloc];
}


- (void)parserDidStartDocument:(NSXMLParser *)parser
{
    parsedItemCounter = 0;
}

// 
- (void)parseXMLFileAtURL:(NSURL *)URL parseError:(NSError **)error
{	
    NSXMLParser *parser = [[NSXMLParser alloc] initWithContentsOfURL:URL];
   
    // if URL contains white space or something else that should be replaced with escape characters
    // then do that.
//    NSString *URLString = [NSString ];
//    NSLog(@"URLString: %@", URLString);
//    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
//    self.URL = [NSURL URLWithString:URLString];
    
    // Keep the URL information in the embedded object for the BBS item
    self.URL = URL;
    
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks.
    [parser setDelegate:self];
    
    // Depending on the XML document you're parsing, you may want to enable these features of NSXMLParser.
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
    [parser parse];
    
    NSError *parseError = [parser parserError];
    if (parseError && error) {
        *error = parseError;
    }
    
    [parser release];
}


- (void)parseXMLWithData:(NSData *)xmlData parseError:(NSError **)error {
//    self.listItemInformationObject = [[NSMutableArray alloc] init]; //  initWithArray:MAX_ITEMS];
    
    NSXMLParser *parser = [[NSXMLParser alloc] initWithData:xmlData];
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks.
    [parser setDelegate:self];
    
    // Depending on the XML document you're parsing, you may want to enable these features of NSXMLParser.
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
    [parser parse];
    
    NSError *parseError = [parser parserError];
    if (parseError && error) {
        NSLog(@"returned with errors");
        //if (parseError != nil) *error = parseError;
    }
    
    [parser release];
    
    // Print the number of total item information added to the internal list
//    NSLog(@"total: %d", [self.listItemInformationObject count]);
}



- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    if (qName) {
        elementName = qName;
    }
    
    // Debug message
//    NSLog(@"elementName = %@", elementName);
//    NSLog(@"qName = %@", qName);

    // If the number of parsed items is greater than MAX_ELEMENTS, abort the parse.
    // Otherwise the application runs very slowly on the device.
    if (parsedItemCounter >= MAX_ITEMS) {
        [parser abortParsing];
    }
    
    if ([elementName isEqualToString:@"bbs"]) {
        
        parsedItemCounter++;
        
        // TODO: adding the object described by this XML entity should be handled here.
        // Assign a new object representing the XML entity.
        // Need to handle this part to collect objects in the server item list view controller
        //self.currentItemObject = [[Item alloc] init];
        //[(id)[[UIApplication sharedApplication] delegate] performSelectorOnMainThread:@selector(addToItemList:) withObject:self.currentItemObject waitUntilDone:YES];
        
        // Debugging purpose
       // NSLog(@"BBS item found: elementName[%@], namespaceURI[%@], qName[%@]", elementName, namespaceURI, qName);
        
        return;
    }
        
    if ([elementName isEqualToString:@"entry"]) {
        [self.logger output:@"entry found" showUpOnlyOnDebugMode:YES];
       // NSString *relAtt = [attributeDict valueForKey:@"rel"];
    }
    else if ([elementName isEqualToString:@"itemName"]) {
        self.contentOfCurrentItemProperty = [NSMutableString string];
    } else if ([elementName isEqualToString:@"shortDescription"]) {
        self.contentOfCurrentItemProperty = [NSMutableString string];
    } else if ([elementName isEqualToString:@"field"]) {
        // Create a mutable string to hold the contents of the 'field' element.
        // The contents are collected in parser:foundCharacters:.
        //NSLog(@"field tag starts");
        self.contentOfCurrentItemProperty = [NSMutableString string];
        
    } else if ([elementName isEqualToString:@"data"]) {
        // Create a mutable string to hold the contents of the 'updated' element.
        //NSLog(@"data tag starts");
        self.contentOfCurrentItemProperty = [NSMutableString string];
    } else {
        // The element isn't one that we care about, so set the property that holds the 
        // character content of the current element to nil. That way, in the parser:foundCharacters:
        // callback, the string that the parser reports will be ignored.
        self.contentOfCurrentItemProperty = nil;
    }
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{     
    if (qName) {
        elementName = qName;
    }
    
    // self.contentOfCurrentItemProperty is filled with the characters collocted in parse:parser:foundCharacters
    // therfore, simply assign the property to the corresponding member of the BBS object
    
    // According to the formation of the original BBS application's xml bbs item,
    // when "entry" tag is found, one field in one BBS item can be generated.
    // Currently, we only have two tags in case of a BBS item: field/value tag.
    // Except the field "itemName" and "shortDescription", value in the field tag will be used as the name of the field.
    // The value in the data tag will be used as the value to be assigned to the above field name.

    // Not like in the item information, there are only tags whose names are "field" and "data".
    // The mandatory field names such as "itemName" and "shortDescription" should be found within the tag "field"
    // and the value for the field should be picked.
    // CHANGED: just use new tags: "itemName" "shortDescription" 
    
    if ([elementName isEqualToString:@"entry"]) {
        //self.currentItemObject.itemName = self.contentOfCurrentItemProperty;
        
    }
    // The following two tags are mandatory in the xml file for an BBS item.
    else if ([elementName isEqualToString:@"itemName"]) {
        NSLog(@"itemName = %@", self.contentOfCurrentItemProperty);
        self.itemName = self.contentOfCurrentItemProperty;
        
    } else if ([elementName isEqualToString:@"shortDescription"]) {
        NSLog(@"shortDescription = %@", self.contentOfCurrentItemProperty);
        self.shortDescription = self.contentOfCurrentItemProperty;
        
    } else if ([elementName isEqualToString:@"fileName"]) { // Not likely to be found
        NSLog(@"fileName = %@", self.contentOfCurrentItemProperty);
    }
    // All rest of fields can be combined with the name given within the "field" tag.
    else if ([elementName isEqualToString:@"field"]) {
        self.currentItemFieldName = self.contentOfCurrentItemProperty;
        
    } else if ([elementName isEqualToString:@"data"]) {
        NSLog(@"fieldName = %@", self.currentItemFieldName);
        NSLog(@"value = %@", self.contentOfCurrentItemProperty);
        [self.fields addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:
            self.contentOfCurrentItemProperty, self.currentItemFieldName,
            nil]];
        
        NSLog(@"current number of items in the member, fields: %d", [self.fields count]);
        
    }
    
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    if (self.contentOfCurrentItemProperty) {
        // If the current element is one whose content we care about, append 'string'
        // to the property that holds the content of the current element.
        [self.contentOfCurrentItemProperty appendString:string];
    }
}

@end
