//
//  LocalItemTableViewController.m
//  BBSApp
//
//  Created by Ki Woon Sung on 7/25/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import "Contants.h"
#import "ItemInformationXMLReader.h"

#import "LocalItemTableViewController.h"
#import "ItemViewController.h"
#import "ItemInformationTableViewCell.h"


@implementation LocalItemTableViewController

@synthesize list;
@synthesize listItemInformation; 
@synthesize activityIndicatorView = _activityIndicatorView;
@synthesize itemViewController;


- (id)initWithStyle:(UITableViewStyle)style {
	if (self = [super initWithStyle:style]) {
		self.title = @"My Items";
        self.tableView.delegate = self;
        self.tableView.dataSource = self;
        self.tableView.sectionHeaderHeight = kSectionHeaderForItemList;
        self.tableView.rowHeight = kLocalItemEntryRowHeight;
        self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
		self.tabBarItem.image = [UIImage imageNamed:@"local.png"];
        self.view.backgroundColor = [UIColor colorWithWhite:50.0 alpha:20.0];
		
		// set the long name shown in the navigation bar
		self.navigationItem.title=@"My Items";
		
		// create a custom navigation bar button and set it to always say "back"
		UIBarButtonItem *temporaryBarButtonItem=[[UIBarButtonItem alloc] init];
		temporaryBarButtonItem.title=@"Back";
		self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
		[temporaryBarButtonItem release];		

        // Start composing the menu list with the appropriate indicator
        // IF NEEDED, spawn a thread to fetch the item list data so that the UI is not blocked while
        // the application connects to the central repository and parses the XML file received.
        // FYI: [NSThread detachNewThreadSelector:@selector(getItemListData toTarget:self withObject:nil];
        //[self showActivityIndicator];
        
        //[self getItemListData]; // for testing

        // the following function is deprecated so as to use the feed form after version 1.03
        //[self generateItemInformationListInLocalDirectory];

        [self uploadLocalItemsToServer];
	}
	return self;
}


// @DEPRECATED
// The desinated output of this function is the XML file to display the item list as in ItemInformation class.
// By doing so, LocalItemTableViewController gets the information for the table entries from the XML file.
// Don't need to generate the real file but the string variable to compose the HTML stream
// Call this every time the view is appeared.
- (void)generateItemInformationListInLocalDirectory {
    NSLog(@"generateItemInformationListInLocalDirectory called");
    // In this version, all local items are stored in the form of the XML which is the same
    // way of storing the items in Java GUI version.
    NSString *xmlString = @"";
    NSString *xmlBodyString = @"";
    NSString *xmlHeaderString = @"<?xml version=\"1.0\" encoding=\"utf-8\"?><bbs name = \"ItemListLocal\">";
    NSString *xmlFooterString = @"</bbs>";
    NSError *parseError = nil;
   
    // Scan the local directory and gather the information.
    
    
    // Keep this code for the future reference
    NSDirectoryEnumerator *direnum = [[NSFileManager defaultManager] 
                                      enumeratorAtPath:NSHomeDirectory()]; //stringByAppendingString:@"/Documents"]]; 
    NSString *pname; 
    NSString *fileName = @"";
    NSLog(@"NSHomeDirectory: %@", NSHomeDirectory());
    //NSError *outError;
    NSString *itemName;
    NSString *shortDescription;
    NSData *xmlTemp;
    ItemXMLReader *localReader;

    while (pname = [direnum nextObject]) 
    { 
        if ([[pname pathExtension] isEqualToString:@"rtfd"]) 
        { 
            /* don’t enumerate this directory */ 
            [direnum skipDescendents]; 
        } 
        else if ([[pname pathExtension] isEqualToString:@"xml"]) 
        { 
            /* ...process file here... */ 
            fileName = [@"/" stringByAppendingString:[pname lastPathComponent]];
            NSLog(@"pname = %@", pname);
            NSLog(@"fileName = %@", fileName);
            
            localReader = [[ItemXMLReader alloc] init];
            NSString *fullPath = [[[NSBundle mainBundle] bundlePath] stringByAppendingString:fileName];
            NSLog(@"fullPath of XML file:%@", fullPath);
            xmlTemp = [[NSData alloc]initWithContentsOfFile:fullPath];
            if (xmlTemp == nil) {
                NSLog(@"xml source data has not beed loaded properly");
            }
            else {
                NSLog(@"xml source data loaded");
                NSError **error;
                [localReader parseXMLWithData:xmlTemp parseError:error];
                itemName = localReader.itemName;
                shortDescription = localReader.shortDescription;
 
                // Data format error handling
                if (itemName == nil) { 
                    itemName = [fileName stringByAppendingString:@" - invalid format"]; 
                }
                if (shortDescription == nil) { 
                    shortDescription = @"description missing";
                }
                
                NSLog(@"generating xml body string for:");
                NSLog(@"itemName = %@", itemName);
                NSLog(@"shortDescription = %@", shortDescription);
                
                xmlBodyString = [xmlBodyString stringByAppendingString:@"<entry><fileName>"];
                xmlBodyString = [xmlBodyString stringByAppendingString:[pname lastPathComponent]];
                xmlBodyString = [xmlBodyString stringByAppendingString:@"</fileName>"];
                xmlBodyString = [xmlBodyString stringByAppendingString:@"<itemName>"];
                xmlBodyString = [xmlBodyString stringByAppendingString:itemName];
                xmlBodyString = [xmlBodyString stringByAppendingString:@"</itemName><shortDescription>"];
                xmlBodyString = [xmlBodyString stringByAppendingString:shortDescription];
                xmlBodyString = [xmlBodyString stringByAppendingString:@"</shortDescription></entry>"];
            
                NSLog(@"xml string generated: %@", [pname lastPathComponent]);
            }
        } 
    } 
    
    // Generate the XML string
    xmlString = [xmlString stringByAppendingString:xmlHeaderString];
    xmlString = [xmlString stringByAppendingString:xmlBodyString];
    xmlString = [xmlString stringByAppendingString:xmlFooterString];
    
    // Should change xmlString to NSData type here.
    NSData *xmlData = [[NSData alloc]initWithData:[xmlString dataUsingEncoding:NSUTF8StringEncoding]];
    
    // Pass the generated XML string to the item information list parser 
    ItemInformationXMLReader *streamingParser = [[ItemInformationXMLReader alloc] init];
    [streamingParser parseXMLWithData:xmlData parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
    else {
        NSLog(@"Item information XML for local items has been parsed correctly");
        self.listItemInformation = [streamingParser getListItemInformation];        
    }
    
    [streamingParser release];   
    
    NSLog(@"self.listItemInformation has %d item information", [self.listItemInformation count]);
    
    // TODO: store sharing status for each item list object
    // this information cannot be obtained when parsing the XML files
    // only by looking at the folder the XML files parsed can determine the status of sharing of each item.

    NSLog(@"starting: sharing status setting");
    
    
    
    NSLog(@"ending: sharing status setting");
    
    // debug info
    NSUInteger count = 0;
    for (ItemInformation *itemInfo in self.listItemInformation) {
        NSLog(@"count: %d", count++);
        NSLog(@"itemName: %@", itemInfo.itemName);
        NSLog(@"shortDes: %@", itemInfo.shortDescription);
    }
    
}


- (void)uploadLocalItemsToServer {
    // this is done in another class: when advertising items
}

// never called yet
- (BOOL)copyDefaultXMLFileToDocumentsFolder:(NSString *)fileName error:(NSError **)outError
{
    NSString *pureFileName = [@"/" stringByAppendingString:[fileName lastPathComponent]];
    NSLog(@"pureFileName: %@", pureFileName);
    
    fileName = [@"/" stringByAppendingString:fileName];
    
    NSArray *paths;
    NSFileManager *mgr = [NSFileManager defaultManager];
    
    paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *sourceFileFullPath = [NSHomeDirectory() stringByAppendingString:fileName];
    NSLog(@"sourceFileFullPath: %@", sourceFileFullPath);
    
    NSString *bundlePath = [[NSBundle mainBundle] bundlePath];
    NSLog(@"bundlePath %@", bundlePath);
    
    if ([paths count] > 0)
    {
        // only copying one file
        NSString *destinationPath = [NSHomeDirectory() stringByAppendingString:@"/Documents"];
        NSString *destinationFileAndPath = [destinationPath stringByAppendingString:pureFileName];
        NSLog(@"destinationPath: %@", destinationPath);
        NSLog(@"destibationFileAndPath: %@", destinationFileAndPath);

        if ([mgr fileExistsAtPath:destinationPath]) {
            NSLog(@"destination path found: %@", destinationPath);
        }
        else {
            [mgr createDirectoryAtPath:destinationPath attributes:nil];
            NSLog(@"new directory: %@", destinationPath);
        }
        
        if (![mgr copyItemAtPath:fileName toPath:destinationFileAndPath error:outError])
        {
            if ([*outError code]) {
                NSLog(@"%@ %d %@ ", [*outError domain], [*outError code], [*outError localizedDescription]);
            }      
            if ([mgr fileExistsAtPath:sourceFileFullPath]) {
                NSLog(@"source path found:%@", sourceFileFullPath);
            }
            
            
            NSLog(@"No");
            return NO;
        }
        return YES;
    }
    // create a suitable NSError object to return in outError
    return NO;
}


- (void)loadView {
    [super loadView];
    
    NSLog(@"LocalItemTableViewController: loadView");
    [self showTableFooter];
    [self displayStatusOnToolbar:@"Last uploaded on ..." showIndicator:NO];
}


#pragma mark UITableViewDelegate methods

- (UITableViewCellAccessoryType)tableView:(UITableView *)tableView accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellAccessoryDisclosureIndicator;
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [listItemInformation count];
}

- (id)objectInListAtIndex:(NSUInteger)theIndex {
    NSLog(@"objectInListAtIndex: %d", theIndex);
    return [listItemInformation objectAtIndex:theIndex];
}

- (void)getList:(id *)objsPtr range:(NSRange)range {
    [listItemInformation getObjects:objsPtr range:range];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	//static NSString *MyIdentifier = @"MyIdentifier";
//	
//	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
//	if (cell == nil) {
//		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
//	}
//	// Configure the cell
//	return cell;
    static NSString *MyIdentifier = @"MyIdentifier";
    NSLog(@"tableView:cellForRowAtIndexPath:indexPath.row=%d", indexPath.row);
    
  	ItemInformationTableViewCell *cell = (ItemInformationTableViewCell *)[tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[ItemInformationTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
        CGRect frame = CGRectMake(0.0, 0.0, 320.0, 47.0);
        cell.backgroundView = [[UIView alloc]initWithFrame:frame];
        
        // MEMO: By doing this, the background color for each row can be defined.
        // Need to initialize cell.backgroundView first.
        if (((indexPath.row)%2)==0) {
            //UIView *myContentView = cell.contentView;
            //myContentView.opaque = YES;
//            myContentView.backgroundColor = [UIColor clearColor];
//            //cell.contentView.backgroundColor = [UIColor blueColor];
//            myContentView.backgroundColor = [UIColor blueColor];
//            [myContentView setOpaque:YES];
//            //myContentView.backgroundColor = [UIColor redColor];
            //cell.backgroundView.backgroundColor = [UIColor lightGrayColor];
        } else {
            //cell.backgroundView.backgroundColor = [UIColor grayColor];
        }
	}
    
	ItemInformation *itemInformationForRow = [listItemInformation objectAtIndex:indexPath.row];
    [cell setItemInformation:itemInformationForRow];
	return cell;
    
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    NSLog(@"LocalItemViewController clicked");
    
	ItemInformation *itemInformation;
    itemInformation = [(ItemInformationTableViewCell *)[tableView cellForRowAtIndexPath:indexPath] getItemInformation];
    NSLog(@"indexPath.row = %d", indexPath.row);
    NSLog(@"selected.itemName = %@", itemInformation.itemName);
    NSLog(@"selected.fileName = %@", itemInformation.fileName);
    
    
    // Only the item information intance which is supposed to contain only the brief information about the BBS
    // item is stored in the cell instance for the table.
    ItemViewController *localItemViewController = [[ItemViewController alloc] initWithItemFileName:itemInformation.fileName itemLocation:BBSItemLocationLocal];
    // Here, set the link information to get the contents.
    // According to the UI guideline, contents are loaded only after the view shows up.
    self.itemViewController = localItemViewController;
    
    [[self navigationController] pushViewController:self.itemViewController animated:YES];
    [localItemViewController release];
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	return @"Notices you have locally";
}


/*- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    CGFloat _height = 50;
    if (indexPath.row == 2) _height = 100;
    return _height;
}*/


- (void)dealloc {
    [listItemInformation release];
    [itemViewController release];

	[super dealloc];
}


- (void)viewDidLoad {
    NSLog(@"LocalItemTableViewController: viewDidLoad");
	[super viewDidLoad];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
    NSLog(@"viewDidAppear");
	[super viewDidAppear:animated];

   // [self hideActivityIndicator];
}

- (void)viewWillDisappear:(BOOL)animated {
}

- (void)viewDidDisappear:(BOOL)animated {
}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}

- (void) showActivityIndicator {
    // Show the activity indicator before parsing the contents.
    UIActivityIndicatorView *localActivityIndicatorView;
    CGRect frame = CGRectMake(kActivityIndicatorXOrigin, kActivityIndicatorYOrigin, kActivityIndicatorSize, kActivityIndicatorSize);
    localActivityIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:frame];
    [localActivityIndicatorView startAnimating];
    localActivityIndicatorView.hidesWhenStopped = YES;
    localActivityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleGray;
    [localActivityIndicatorView sizeToFit];
    localActivityIndicatorView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
                                                   UIViewAutoresizingFlexibleRightMargin |
                                                   UIViewAutoresizingFlexibleTopMargin |
                                                   UIViewAutoresizingFlexibleBottomMargin);
    
    self.activityIndicatorView = localActivityIndicatorView;
    
    [self.view addSubview:self.activityIndicatorView];
    
}


- (void)hideActivityIndicator {
    if (self.activityIndicatorView != nil) {
        [self.activityIndicatorView stopAnimating];
    }
    
}


- (void)showTableHeader {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 40); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor blueColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont boldSystemFontOfSize:18]; 
    tableTitle.text = @"Name"; 
    self.tableView.tableHeaderView = tableTitle; 
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    

- (void)showTableFooter {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 45); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor grayColor]; 
//    tableTitle.shadowColor = [UIColor lightGrayColor];
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont fontWithName:@"Arial" size:11]; 
    tableTitle.text = @"Copyright 2008 by Columbia University";
    tableTitle.textAlignment = UITextAlignmentCenter;
    
    self.tableView.tableFooterView = tableTitle; 
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    


- (void)displayStatusOnToolbar:(NSString *)text showIndicator:(BOOL)show {
    CGRect infoRect = CGRectMake(0.0, 0.0, 150, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(1.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = text; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// create a refresh tool bar button for future use
	UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                 target:self 
                                                                                 action:@selector(refresh:)];
	
    // Create an activity indicator at the rightmost of the toolbar
    UIActivityIndicatorView *activityIndicator;
    CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
    activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
    activityIndicator.opaque = YES;
    activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
    [activityIndicator startAnimating];
    
    UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
    [activityIndicator release];
    
	NSArray *items;
    if (show) {
        items = [NSArray arrayWithObjects: indicatorItem, systemItem, flexItem, refreshItem, nil];
    } else {
        items = [NSArray arrayWithObjects: systemItem, flexItem, refreshItem, nil];
    }
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
	[refreshItem release];
}




@end

