//
//  PeerItemTableViewController.m
//  BBSApp
//
//  Created by Kiwoon Sung on 8/1/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <SystemConfiguration/SystemConfiguration.h>

#import "Contants.h"

#import "PeerItemTableViewController.h"

#import "BBSAppAppDelegate.h"
#import "BBSItemLocation.h"

#import "ItemInformationTableViewCell.h"
#import "ItemViewController.h"

#import "ItemXMLReader.h"
#import "ItemInformationXMLReader.h"

// Imports for the internal Web server
#import "defs.h"

@implementation PeerItemTableViewController

@synthesize list;
@synthesize tableViewMember;

@synthesize listItemInformation; // THIS is a mutable array list keeping the item information objects acquired from the server
@synthesize activityIndicatorView = _activityIndicatorView;
@synthesize itemViewController;
@synthesize receivedData = _receivedData;

// Added
@synthesize netService;


// About to be deprecated
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		// Initialization code
        self.title = @"Repository";
        
        // Setup the tab bar image: Temporary use since it's for Downloads
		self.tabBarItem.image = [UIImage imageNamed:@"repository.png"]; //UITabBarSystemItemDownloads; //
		
		// set the long name shown in the navigation bar
		self.navigationItem.title=@"Bulletin Board";
        
        // Testing
        list = [[NSMutableArray alloc]initWithCapacity:2];
        [self generateTableEntries];
        
        // create a custom navigation bar button and set it to always say "back"
		UIBarButtonItem *temporaryBarButtonItem=[[UIBarButtonItem alloc] init];
		temporaryBarButtonItem.title=@"Nodes";
		self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
		[temporaryBarButtonItem release];
        
        // Start composing the menu list with the appropriate indicator
        // IF NEEDED, spawn a thread to fetch the item list data so that the UI is not blocked while
        // the application connects to the central repository and parses the XML file received.
        // FYI: [NSThread detachNewThreadSelector:@selector(getItemListData toTarget:self withObject:nil];
        [self showActivityIndicator];
        
        didLaunchHTTPDaemon = NO;
        
        //[self getItemListData]; // for testing
        //        [self generateItemInformationListByXML];
	}
	return self;
}


- (id)initWithNetService:(NSNetService *)_netService {
    if (self = [super initWithNibName:nil bundle:nil]) {
        self.title = @"Peer items";
        self.navigationItem.title = @"Bulletin Board";
        
        // Keep information about the peer to which user wants to connect
        self.netService = _netService;
        
        // Create back button
        UIBarButtonItem *temporaryBarButtonItem = [[UIBarButtonItem alloc]init];
        temporaryBarButtonItem.title = @"Back";
        self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
        [temporaryBarButtonItem release];
        
    }
    return self;
}


- (void)loadView {
    //    CGRect screenRect = [[UIScreen mainScreen] applicationFrame];
	
	// setup our parent content view and embed it to your view controller
	//
	UIView *contentView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
    contentView.autoresizesSubviews = YES;
	contentView.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
	contentView.backgroundColor = [UIColor groupTableViewBackgroundColor];	// use the table view background color
	self.view = contentView;
	[contentView release];
	
	// create the UIToolbar at the bottom of the view controller
	//
	toolbar = [UIToolbar new];
	toolbar.barStyle = UIBarStyleBlackTranslucent;
	
	// size up the toolbar and set its frame
	[toolbar sizeToFit];
	CGFloat toolbarHeight = [toolbar frame].size.height - 20.0;
	CGRect mainViewBounds = self.view.bounds;
    //    CGFloat toolbarYOrigin = self.navigationController.navigationBar.frame.size.height;
    CGFloat toolbarYOrigin = 0.0;
    NSLog(@"toobarYOrigin = %f", toolbarYOrigin);
	[toolbar setFrame:CGRectMake(0.0, //CGRectGetMinX(mainViewBounds),
								 toolbarYOrigin, //CGRectGetMinY(mainViewBounds) + CGRectGetHeight(mainViewBounds) - (toolbarHeight * 2.0) + 2.0,
								 CGRectGetWidth(mainViewBounds),
								 toolbarHeight)];
	
	[self.view addSubview:toolbar];
	
    //	currentSystemItem = UIBarButtonSystemItemDone;
	[self createToolbarItems];
    
    // Then create a table view. This will contain a scroll view in itself by default
    CGRect tableViewRect = CGRectMake(0.0, 
                                      toolbarYOrigin + toolbarHeight, 
                                      CGRectGetWidth(mainViewBounds), 
                                      CGRectGetHeight(mainViewBounds) - toolbarHeight - toolbarYOrigin - 93);
    tableViewMember = [[UITableView alloc]initWithFrame:tableViewRect style:UITableViewStylePlain];
    tableViewMember.delegate = self;
    tableViewMember.dataSource = self;
    
    [self showTableFooter];
    
    [self.view addSubview:tableViewMember];
    
    NSLog(@"PeerItemTableViewController: loadView exit");
}


- (void)viewDidLoad {
    [super viewDidLoad];
    
    NSLog(@"PeerItemTableViewController: viewDidLoad");
    [self displayStatusOnToolbar:@"Hello" showIndicator:NO showStopButton: NO showRefreshButton:YES];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
    
    [self hideActivityIndicator];
    [self refreshTable];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}


- (void)dealloc {
	[super dealloc];
}


#pragma mark UITableView delegate methods

// decide what kind of accesory view (to the far right) we will use
- (UITableViewCellAccessoryType)tableView:(UITableView *)tableView accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellAccessoryDisclosureIndicator;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [listItemInformation count];
}

- (id)objectInListAtIndex:(NSUInteger)theIndex {
    NSLog(@"objectInListAtIndex: %d", theIndex);
    return [listItemInformation objectAtIndex:theIndex];
}

- (void)getList:(id *)objsPtr range:(NSRange)range {
    [listItemInformation getObjects:objsPtr range:range];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *MyIdentifier = @"MyIdentifier";
    NSLog(@"tableView:cellForRowAtIndexPath:indexPath.row=%d", indexPath.row);
    
  	ItemInformationTableViewCell *cell = (ItemInformationTableViewCell *)[tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[ItemInformationTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
	}
    
	ItemInformation *itemInformationForRow = [listItemInformation objectAtIndex:indexPath.row];
    [cell setItemInformation:itemInformationForRow];
	return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    NSLog(@"PeerItemTableViewController clicked [row, section] = [%d, %d]", indexPath.row, indexPath.section);
    
	ItemInformation *itemInformation;
    itemInformation = [(ItemInformationTableViewCell *)[tableView cellForRowAtIndexPath:indexPath] getItemInformation];
    NSLog(@"indexPath.row = %d", indexPath.row);
    NSLog(@"selected.itemName = %@", itemInformation.itemName);
    NSLog(@"selected.fileName = %@", itemInformation.fileName);
    
    // Workaround:
    // If the program sends a request to itself, the internal web server cannot have a chance to
    // response to it since logic is blocked until XML parser loads all data from peer (itself in 
    // this case). So, this is a workaround to handle that situation. 
    // If the requesting node is same to itself, find the item in the local directory.
    BBSAppAppDelegate *appDelegate = [[UIApplication sharedApplication]delegate];
    NSString *registeredName = [[NSString alloc]initWithString:[appDelegate getSelfRegisteredName]];

    ItemViewController *localItemViewController;
    if ([registeredName isEqualToString:[netService name]]) {
        NSLog(@"find this item in local to reduce the amount of networking");
        localItemViewController = [[ItemViewController alloc] initWithItemFileName:itemInformation.fileName itemLocation:BBSItemLocationLocal];
    } else {
        localItemViewController = [[ItemViewController alloc] initWithNetService:netService fileName:itemInformation.fileName];
    }
    
    self.itemViewController = localItemViewController;
    [[self navigationController] pushViewController:self.itemViewController animated:YES];
    [localItemViewController release];
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    NSString *headerTitle = @"";
    headerTitle = [self.netService name];
    return headerTitle;
}


#pragma mark UITableViewDelegate Ends


#pragma mark NSURLConnection delegate


- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse 
                                                                     *)response 
{ 
    [self.receivedData setLength:0]; 
} 


- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [self.receivedData appendData:data];
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    [connection release];
    // receivedData is declared as a method instance elsewhere
    [self.receivedData release];
    
    // inform the user
    NSLog(@"connection failed: error - %@ %@",
          [error localizedDescription],
          [[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
    
    [self displayStatusOnToolbar:[error localizedDescription] showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    // receivedData is declared as a method instance elsewhere
    NSLog(@"succeeded: received %d bytes of data",[self.receivedData length]);
    
    [self reloadItemInformationList:self.receivedData];
    
    // release the connection, and the data object
    [connection release];
    [self.receivedData release];
    
    [tableViewMember reloadData];
    NSDate *now = [[NSDate alloc] init];
    NSString *label = @"";
//    NSTimeZone *timeZoneEST = [NSTimeZone timeZoneWithName:@"EST"];
    // this is deprecated method
    //label = [now descriptionWithCalendarFormat:@"HH:MM:SS" timeZone:timeZoneEST locale:[NSLocale currentLocale]];
    label = [label stringByAppendingFormat:@"Updated on %@", [now description]];
    [self displayStatusOnToolbar:label showIndicator:NO showStopButton:NO showRefreshButton:YES];

}


#pragma mark NSURLConnection delegate End

#pragma mark User defined functions

- (void)createToolbarItems
{	
    CGRect infoRect = CGRectMake(0.0, 0.0, 150, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(1.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = @"Latest update "; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// create a refresh tool bar button for future use
	UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                 target:self 
                                                                                 action:@selector(refreshAction:)];
	
    // Create an activity indicator at the rightmost of the toolbar
    UIActivityIndicatorView *activityIndicator;
    CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
    activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
    activityIndicator.opaque = YES;
    activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
    [activityIndicator startAnimating];
    
    UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
    [activityIndicator release];
	
	NSArray *items = [NSArray arrayWithObjects: indicatorItem, systemItem, flexItem, refreshItem, nil];
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
	[refreshItem release];
}


- (void)refreshTable {
    [self displayStatusOnToolbar:@"Refreshing" showIndicator:YES showStopButton:NO showRefreshButton:NO];
    
    [self generateItemListInformationByURL];
}


- (void)refreshAction:(id)sender {
    NSLog(@"refresh button clicked");
    [self displayStatusOnToolbar:@"Refreshing" showIndicator:YES showStopButton:NO showRefreshButton:NO];
    
    [self generateItemListInformationByURL];
}


- (void)stopAction:(id)sender {
    NSLog(@"stop button clicked");
    
    [self displayStatusOnToolbar:@"Stopped" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)generateTableEntries {
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"iPhone For Sale", @"itemName",
                     nil]];
    
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"BBS Item For Testing", @"itemName",
                     nil]];
    
}    


- (void) showActivityIndicator {
    // Show the activity indicator before parsing the contents.
    UIActivityIndicatorView *localActivityIndicatorView;
    CGRect frame = CGRectMake(kActivityIndicatorXOrigin, kActivityIndicatorYOrigin, kActivityIndicatorSize, kActivityIndicatorSize);
    localActivityIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:frame];
    [localActivityIndicatorView startAnimating];
    localActivityIndicatorView.hidesWhenStopped = YES;
    localActivityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleGray;
    [localActivityIndicatorView sizeToFit];
    localActivityIndicatorView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
                                                   UIViewAutoresizingFlexibleRightMargin |
                                                   UIViewAutoresizingFlexibleTopMargin |
                                                   UIViewAutoresizingFlexibleBottomMargin);
    
    self.activityIndicatorView = localActivityIndicatorView;
    
    [self.view addSubview:self.activityIndicatorView];
    
}


- (void)hideActivityIndicator {
    [self.activityIndicatorView stopAnimating];
    
}


- (void)showTableHeader {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 40); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor blueColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont boldSystemFontOfSize:18]; 
    tableTitle.text = @"Name"; 
    
    tableViewMember.tableFooterView = tableTitle;
    
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    


- (void)showTableFooter {
    // See if anything worthy
    CGRect titleRect = CGRectMake(0, 50, 300, 45); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor lightGrayColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont fontWithName:@"Arial" size:11]; 
    tableTitle.text = @"Copyright 2008 by Columbia University";
    tableTitle.textAlignment = UITextAlignmentCenter;
    
    tableViewMember.tableFooterView = tableTitle;
    
    [tableTitle release];
    
}    


// URL in this method should be acquired from the server information class
// Don't need to pass URL information to this method without a reason
// MEMO: If you request a file by URL, it blocks the execute.
// It is okay in most cases but it's not when it sends a request to http in itself.
- (void)generateItemListInformationByURL {
    
    //    NSError *parseError = nil;
    NSLog(@"PeerItemTableViewController:generateItemListInformationByURL called");
    
    // Create peer connection information
//    BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
//    NSString *URLString = [appDelegate getServerURLString];
    
    // MEMO: In some case (actually for sure), the host name obtained from the instance netService
    // may not be used as a real host name if you connect to them via http.
    // In other words, if you define the base URL for the request based on the host name given by
    // netService, the URL cannot reach to that host. So far, this issue occurred on iPod Touch
    // running iPhoneOS 2.0. So, it is needed to see if the destination is iPod Touch or not.
    // No other way I have found except inspecting addresses field in netService.
    // Usually, those device only has one address in it and broadcast the information to
    // Zeroconf.
    NSString *addressString = @"";
    NSArray *address = [netService addresses];
    NSLog(@"count=%d", [address count]);
    if ([address count] == 0) {
        NSLog(@"internal error: no ip address for host: %@", [netService hostName]);
    }
    
    if ([address count] == 1) {
        // Assume that this occurrs only when the devices such as iPhone/iPod Touch
        // are subject to connect to.
        NSLog(@"Seems like iPod Touch");
        
        struct sockaddr_in addrIn = *(struct sockaddr_in *)[[address objectAtIndex:0] bytes];
        addressString = [NSString stringWithFormat: @"%s", inet_ntoa(addrIn.sin_addr)];
        
        NSLog(@"addressString=%@", addressString);
    }
    
    NSString *URLString = @"http://";
    
    // Handle in a different way in case that iPod is about to be connected
    if ([addressString isEqualToString:@""]) {
        URLString = [@"http://" stringByAppendingString:[netService hostName]];
    } else {
        URLString = [@"http://" stringByAppendingString:addressString];
    }
    
    URLString = [URLString stringByAppendingFormat:@":%d/", [self.netService port]];

    NSLog(@"try to get the item information from \"%@\"", URLString);
    
    // Adding percent escapes
    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    NSLog(@"URL request after percent sequence adding: %@", URLString);
    
    
    NSString *post = @"registered_name=test&command=iteminformationlist";
    NSData *postData = [post dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
    
    NSString *postLength = [NSString stringWithFormat:@"%d", [postData length]];
    
    NSMutableURLRequest *request = [[[NSMutableURLRequest alloc] init] autorelease];
    URLString = [URLString stringByAppendingString:@"BBS"];
    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    [request setURL:[NSURL URLWithString:URLString]];
    [request setHTTPMethod:@"POST"];
    [request setValue:postLength forHTTPHeaderField:@"Content-Length"];
    [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPBody:postData];
    [request setTimeoutInterval:_REQUEST_TIME_OUT_IN_SEC];
    [request setCachePolicy:NSURLRequestUseProtocolCachePolicy];
    
    // Asynchronous send
    NSURLConnection *theConnection=[[NSURLConnection alloc] initWithRequest:request 
                                                                   delegate:self]; 

    if (theConnection) { 
        self.receivedData = [[NSMutableData data] retain]; 
    } else { 
        // inform the user that the download could not be made 
        NSLog(@"error on requesting");
        
        [self displayStatusOnToolbar:@"Failed to request information" showIndicator:NO showStopButton:NO showRefreshButton:YES];
    } 
    
    
}


- (void)reloadItemInformationList:(NSData *)receivedData {
    NSError *parseError = nil;
    
    ItemInformationXMLReader *streamingParser = [[ItemInformationXMLReader alloc] init];
    //    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    [streamingParser parseXMLWithData:receivedData parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
    else {
        NSLog(@"Item information XML has been parsed correctly");
        //self.listItemInformation = [[NSMutableArray alloc] initWithArray:[streamingParser getListItemInformation] copyItems:YES];
        self.listItemInformation = [streamingParser getListItemInformation];        
    }
    
    [streamingParser release];   
    
    NSLog(@"self.listItemInformation has %d item information", [self.listItemInformation count]);
    
    // debug info
    NSUInteger count = 0;
    for (ItemInformation *itemInfo in self.listItemInformation) {
        NSLog(@"count: %d", count++);
        NSLog(@"fileName: %@", itemInfo.fileName);
        NSLog(@"itemName: %@", itemInfo.itemName);
        NSLog(@"shortDes: %@", itemInfo.shortDescription);
    }
}


- (void)displayStatusOnToolbar:(NSString *)text 
                 showIndicator:(BOOL)showIndicator
                showStopButton:(BOOL)showStopButton
             showRefreshButton:(BOOL)showRefreshButton 
{
    CGRect infoRect = CGRectMake(0.0, 0.0, STATUS_ON_TOOLBAR_WIDTH, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(0.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = text; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// Create and add an indicator if needed
	NSMutableArray *items = [[NSMutableArray alloc]init];
    if (showIndicator) {
        // Create an activity indicator at the rightmost of the toolbar
        UIActivityIndicatorView *activityIndicator;
        CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
        activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
        activityIndicator.opaque = YES;
        activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
        [activityIndicator startAnimating];
        
        UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
        
        [items addObject:indicatorItem];
        
        [activityIndicator release];
    }
    
    // Mandatory field: add systemItem
    [items addObject:systemItem];
    [items addObject:flexItem];
    
    // Create and add a stop button
    if (showStopButton) {
        // Create a stop tool bar button
        UIBarButtonItem *stopItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop 
                                                                                  target:self 
                                                                                  action:@selector(stopAction:)];
        
        [items addObject:stopItem];
        [stopItem release];
    }
    
    // Create and add a refresh button if needed
    if (showRefreshButton) {
        // Create a refresh tool bar button for future use
        UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                     target:self 
                                                                                     action:@selector(refreshAction:)];
        
        [items addObject:refreshItem];
        [refreshItem release];
    }
    
    // Construct the tool bar
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
}


#pragma mark User defined methods END


@end
