//
//  ServerItemTableViewController.m
//  BBSApp
//
//  Created by Ki Woon Sung on 7/25/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <SystemConfiguration/SystemConfiguration.h>

#import "BBSAppAppDelegate.h"
#import "Contants.h"
#import "BBSItemLocation.h"

#import "ItemInformationTableViewCell.h"
#import "ItemViewController.h"
#import "ServerItemTableViewController.h"

#import "ItemXMLReader.h"
#import "ItemInformationXMLReader.h"


@implementation ServerItemTableViewController

@synthesize list; // this is the list that is used for composing menu list of the table view controller
@synthesize listItemInformation; // THIS is a mutable array list keeping the item information objects acquired from the server
@synthesize activityIndicatorView = _activityIndicatorView;
@synthesize itemViewController;
@synthesize containerView;

- (id)initWithStyle:(UITableViewStyle)style {
	if (self = [super initWithStyle:style]) {
		self.title = @"Repository";
        
		self.tabBarItem.image = [UIImage imageNamed:@"repository.png"];
		
		// set the long name shown in the navigation bar
		self.navigationItem.title=@"Bulletin Board";
		
		// create a custom navigation bar button and set it to always say "back"
		UIBarButtonItem *temporaryBarButtonItem=[[UIBarButtonItem alloc] init];
		temporaryBarButtonItem.title=@"Back";
		self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
		[temporaryBarButtonItem release];
        
        // Start composing the menu list with the appropriate indicator
        // IF NEEDED, spawn a thread to fetch the item list data so that the UI is not blocked while
        // the application connects to the central repository and parses the XML file received.
        // FYI: [NSThread detachNewThreadSelector:@selector(getItemListData toTarget:self withObject:nil];
        [self showActivityIndicator];
        list = [[NSMutableArray alloc] init];

        //[self getItemListData]; // for testing
        [self generateItemInformationListByXML];
    }
	return self;
}

- (void)loadView {
    [super loadView];
    
    // Need to initialize a new view and a view for the original view.
    // This is because this controller originally written for working as a table view.
    // But, for the application logics, needed to be modified to work as a general view
    // in order to set a toolbar at the top of the user screen which is not moved even when
    // the user scroll the screen down or up.
    UIView *originalView = [[UIView alloc]initWithFrame:self.view.bounds];
    originalView = self.view;
    
    UITableView *originalTableView = [[UITableView alloc]initWithFrame:self.view.bounds];
    originalTableView = self.tableView;
    
    UIView *newView = [[UIView alloc]initWithFrame:self.view.bounds];
    // Here, change the original view to one that is not moving.
    // The fixed toolbar should be added to this view and the view works as a super view of the table view
    // that is supposed to move when user scrolls.
    self.view = newView;
    
    // create the UIToolbar at the top of the view controller
	toolbar = [UIToolbar new];
	toolbar.barStyle = UIBarStyleDefault;
	
	// size up the toolbar and set its frame
	[toolbar sizeToFit];
	CGFloat toolbarHeight = [toolbar frame].size.height * 5;
	CGRect mainViewBounds = self.view.bounds;
	[toolbar setFrame:CGRectMake(0.0, //CGRectGetMinX(mainViewBounds),
								 0.0, //CGRectGetMinY(mainViewBounds) + CGRectGetHeight(mainViewBounds) - (toolbarHeight * 2.0) + 2.0,
								 CGRectGetWidth(mainViewBounds),
								 toolbarHeight)];
	
	[self.view addSubview:toolbar];
    
    [self showTableFooter];
    originalTableView = self.tableView;

    [originalView addSubview:originalTableView];
    [self.view addSubview:originalView]; // Now the original table view is added to the new fixed view
    
    //self.tableView = originalView;

    
    // Create the container view for this controller. This will be used as a container for the table view and other views.
    //CGRect viewRectFull = [[UIScreen mainScreen] applicationFrame];
    //CGRect viewRectFull = self.view.bounds;
    /*CGRect viewRectFull = CGRectMake(50.0, 50.0, 300.0, 200.0);
    
    //    UIView *localContainerView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
    UIScrollView *localContainerView = [[UIScrollView alloc] initWithFrame:viewRectFull];
    localContainerView.pagingEnabled = NO;
    localContainerView.contentSize = CGSizeMake(localContainerView.frame.size.width, localContainerView.frame.size.height);
    localContainerView.showsVerticalScrollIndicator = YES;
    localContainerView.scrollsToTop = NO;
    localContainerView.delegate = self;
    localContainerView.autoresizesSubviews = YES;
    localContainerView.backgroundColor = [UIColor redColor];
    
    self.containerView = localContainerView;
    [localContainerView release];
    
    */
    // Change the main view
    //self.tableView = self.view;
    //self.view = self.containerView;
    
   
    
  //  [self.view addSubview:containerView];
    
    // Initialize the table view and assign it to the container view as a sub
    // Initialize the table view
    /*self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.sectionHeaderHeight = kSectionHeaderForItemList;
    self.tableView.rowHeight = kItemEntryRowHeight;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    
    [self.view addSubview:self.tableView];
    */
    
    
    NSLog(@"ServerItemTableViewController: loadView");
}


- (void)viewDidLoad {
	[super viewDidLoad];
    
    NSLog(@"ServerItemTableViewController: viewDidLoad");
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
    [self hideActivityIndicator];

    // TODO: 
    // also need to display the network activity indicator on the top of the screen
    // [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    
    // THESE are only for trial
    // send URL synchronous request
    // [self sendHTTPRequestSync];
    //[self generateItemInstanceByXMLTest];
    
    // Remove the network activity indicator.
    // refer to the above code
    
    
}

- (void)viewWillDisappear:(BOOL)animated {
}

- (void)viewDidDisappear:(BOOL)animated {
}

- (void) showActivityIndicator {
    // Show the activity indicator before parsing the contents.
    UIActivityIndicatorView *localActivityIndicatorView;
    CGRect frame = CGRectMake(kActivityIndicatorXOrigin, kActivityIndicatorYOrigin, kActivityIndicatorSize, kActivityIndicatorSize);
    localActivityIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:frame];
    [localActivityIndicatorView startAnimating];
    localActivityIndicatorView.hidesWhenStopped = YES;
    localActivityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleGray;
    [localActivityIndicatorView sizeToFit];
    localActivityIndicatorView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
                                              UIViewAutoresizingFlexibleRightMargin |
                                              UIViewAutoresizingFlexibleTopMargin |
                                              UIViewAutoresizingFlexibleBottomMargin);

    self.activityIndicatorView = localActivityIndicatorView;

    [self.view addSubview:self.activityIndicatorView];

}


- (void)hideActivityIndicator {
    [self.activityIndicatorView stopAnimating];

}


- (void)showTableHeader {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 40); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor blueColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont boldSystemFontOfSize:18]; 
    tableTitle.text = @"Name"; 
    self.tableView.tableHeaderView = tableTitle; 
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    

- (void)showTableFooter {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 45); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor lightGrayColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont fontWithName:@"Arial" size:11]; 
    tableTitle.text = @"Copyright 1998-2008 by Columbia University";
    tableTitle.textAlignment = UITextAlignmentCenter;

    self.tableView.tableFooterView = tableTitle; 
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    

#pragma mark UITableView delegate methods

// decide what kind of accesory view (to the far right) we will use
- (UITableViewCellAccessoryType)tableView:(UITableView *)tableView accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellAccessoryDisclosureIndicator;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [listItemInformation count];
}

- (id)objectInListAtIndex:(NSUInteger)theIndex {
    NSLog(@"objectInListAtIndex: %d", theIndex);
    return [listItemInformation objectAtIndex:theIndex];
}

- (void)getList:(id *)objsPtr range:(NSRange)range {
    [listItemInformation getObjects:objsPtr range:range];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	//static NSString *MyIdentifier = @"MyIdentifier";
//	
//	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
//	if (cell == nil) {
//		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
//	}
//	// Configure the cell
//    cell.text = [[list objectAtIndex:indexPath.row] objectForKey:@"name"];
//	
//
//	return cell;
    
	static NSString *MyIdentifier = @"MyIdentifier";
    NSLog(@"tableView:cellForRowAtIndexPath:indexPath.row=%d", indexPath.row);
    
  	ItemInformationTableViewCell *cell = (ItemInformationTableViewCell *)[tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[ItemInformationTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
	}
    
    // If the information isn't accessible (which could happen if the network isn't available), show an informative
    // message in the first row of the table.
	if ([self isDataSourceAvailable] == NO) {
        cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:@"DefaultTableViewCell"] autorelease];
        cell.text = @"Host Not Available";
		cell.textColor = [UIColor colorWithWhite:0.5 alpha:0.5];
		cell.accessoryType = UITableViewCellAccessoryNone;
		return cell;
	}
	
	ItemInformation *itemInformationForRow = [listItemInformation objectAtIndex:indexPath.row];
    [cell setItemInformation:itemInformationForRow];
	return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    NSLog(@"ServerItemTableViewController clicked");
    
	ItemInformation *itemInformation;
    itemInformation = [(ItemInformationTableViewCell *)[tableView cellForRowAtIndexPath:indexPath] getItemInformation];
    NSLog(@"indexPath.row = %d", indexPath.row);
    NSLog(@"selected.itemName = %@", itemInformation.itemName);
    NSLog(@"selected.fileName = %@", itemInformation.fileName);
    
    
    // Only the item information intance which is supposed to contain only the brief information about the BBS
    // item is stored in the cell instance for the table.
    // Therefore, downloading the corresponding xml file (the name is given in the item information instance
    // should be done before passing the contents to the item view (table type) controller
    ItemViewController *localItemViewController = [[ItemViewController alloc] initWithItemFileName:itemInformation.fileName itemLocation:BBSItemLocationServer];
    // Here, set the link information to get the contents.
    // According to the UI guideline, contents are loaded only after the view shows up.
//    [itemViewController loadItem:itemInformation.fileName]; // only file name is needed
//    itemTableViewController.itemInformation = itemInformation;
    
    self.itemViewController = localItemViewController;
    [[self navigationController] pushViewController:self.itemViewController animated:YES];
    [localItemViewController release];
//    return nil;
    
    // Refer
    // deselect the new row using animation
//    [tableView deselectRowAtIndexPath:newIndexPath animated:YES];
//	
//	// get the element that is represented by the selected row.
//	AtomicElement *element = [dataSource atomicElementForIndexPath:newIndexPath];
//	
//	// create an AtomicElementViewController. This controller will display the full size tile for the element
//	AtomicElementViewController *elementController = [[AtomicElementViewController alloc] init];
//    
//	// set the element for the controller
//	elementController.element = element;
//	
//	// push the element view controller onto the navigation stack to display it
//	[[self navigationController] pushViewController:elementController animated:YES];
//	[elementController release];
   
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	return @"Notices in server (ip add)";
}



/*
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
	
	if (editingStyle == UITableViewCellEditingStyleDelete) {
	}
	if (editingStyle == UITableViewCellEditingStyleInsert) {
	}
}
*/
/*
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}
*/
/*
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/
/*
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}
*/

#pragma mark UITableViewDelegate End

- (void)dealloc {
    [listItemInformation release];
    [self.itemViewController release];
    [list release];
    
	[super dealloc];
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}

#pragma mark User defined functions

- (void)sendHTTPRequestSync {
    NSURL *pathURL;
    NSError *error;
    NSData *searchData;
    NSURLResponse *response;
    
    pathURL = [NSURL URLWithString:@"http://www.apple.com"];
    NSURLRequest *request = [NSURLRequest requestWithURL:pathURL 
                             cachePolicy:NSURLRequestReturnCacheDataElseLoad timeoutInterval:_REQUEST_TIME_OUT_IN_SEC];

    searchData = [NSURLConnection sendSynchronousRequest:request 
                   returningResponse:&response error:&error];
    if ([error code]) {
        NSLog(@"%@ %d %@ %@ %@", [error domain], [error code], [error localizedDescription], request, @"file://localhost/etc/gettytab");
    }        
    
    if (searchData == nil) {
        NSLog(@"no data received");
    }
    else {
        NSLog(@"data transmitted");
        if ([searchData writeToFile:@"./temp.txt" atomically:NO]) {
            NSLog(@"file generated");
        }
        else {
            NSLog(@"file error");
        }
    }
    
    
//    [pathURL release];
//    [error release];
//    [searchData release];
//    [response release];
}

- (void)generateItemInstanceByXMLTest {
    NSError *parseError = nil;
    NSLog(@"generateItemInstanceByXMLTest");

    ItemXMLReader *streamingParser = [[ItemXMLReader alloc] init];
    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:@"http://192.168.1.4/iPhoneSale.xml"] parseError:&parseError];
    [streamingParser release];   
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
}


- (void)generateItemInformationListByXML {
    NSError *parseError = nil;
    NSLog(@"generateItemInformationListByXML called");
    
    // Retrieve the server information
    // Set up the cell.
	BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    // This is the code written before the modification on the server side.
    // Should request the item information list to the server first,
    // and then this will be able to get the file like ItemInformationListFromServer.xml.
    NSString *URLString = [appDelegate getServerURLString];
    URLString = [URLString stringByAppendingString:@"ItemInformationListFromServer.xml"];
    NSLog(@"try to get the item information from \"%@\"", URLString);
    
    
    ItemInformationXMLReader *streamingParser = [[ItemInformationXMLReader alloc] init];
    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
    else {
        NSLog(@"Item information XML has been parsed correctly");
        //self.listItemInformation = [[NSMutableArray alloc] initWithArray:[streamingParser getListItemInformation] copyItems:YES];
        self.listItemInformation = [streamingParser getListItemInformation];        
    }
    
    [streamingParser release];   

    NSLog(@"self.listItemInformation has %d item information", [self.listItemInformation count]);
    
    // debug info
    NSUInteger count = 0;
    for (ItemInformation *itemInfo in self.listItemInformation) {
        NSLog(@"count: %d", count++);
        NSLog(@"itemName: %@", itemInfo.itemName);
        NSLog(@"shortDes: %@", itemInfo.shortDescription);
    }
}


- (void)getItemListData {
    // TODO:
    // 1. connect to the central repository according to the information given by ZeroConf
    // 2. download the XML file describing the item list
    // 3. parse them and generate the item list data object list or something else
    // TEMPORARY:
    // Before getting there, use the fake data
    
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"iPhone For Sale", @"itemName",
                     [listItemInformation objectAtIndex:0], @"itemInformation",
                     nil]];
    
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"BBS Item For Testing", @"itemName",
                     [listItemInformation objectAtIndex:1], @"itemInformation",
                     nil]];
     
    
}


- (BOOL)isDataSourceAvailable {
    static BOOL checkNetwork = YES;
    if (checkNetwork) { // Since checking the reachability of a host can be expensive, cache the result and perform the reachability check once.
        checkNetwork = NO;
        
        Boolean success;    
        const char *host_name = "www.apple.com"; // THIS IS FOR TESTING! Change the value acquired from ZeronConf later
        
        SCNetworkReachabilityRef reachability = SCNetworkReachabilityCreateWithName(NULL, host_name);
        SCNetworkReachabilityFlags flags;
        success = SCNetworkReachabilityGetFlags(reachability, &flags);
        _isDataSourceAvailable = success && (flags & kSCNetworkFlagsReachable) && !(flags & kSCNetworkFlagsConnectionRequired);
        CFRelease(reachability);
    }
    return _isDataSourceAvailable;
}

@end

//NSXMLParserInternalError = 1,
//NSXMLParserOutOfMemoryError = 2,
//NSXMLParserDocumentStartError = 3,
//NSXMLParserEmptyDocumentError = 4,
//NSXMLParserPrematureDocumentEndError = 5,
//NSXMLParserInvalidHexCharacterRefError = 6,
//NSXMLParserInvalidDecimalCharacterRefError = 7,
//NSXMLParserInvalidCharacterRefError = 8,
//NSXMLParserInvalidCharacterError = 9,
//NSXMLParserCharacterRefAtEOFError = 10,
//NSXMLParserCharacterRefInPrologError = 11,
//NSXMLParserCharacterRefInEpilogError = 12,
//NSXMLParserCharacterRefInDTDError = 13,
//NSXMLParserEntityRefAtEOFError = 14,
//NSXMLParserEntityRefInPrologError = 15,
//NSXMLParserEntityRefInEpilogError = 16,
//NSXMLParserEntityRefInDTDError = 17,
//NSXMLParserParsedEntityRefAtEOFError = 18,
//NSXMLParserParsedEntityRefInPrologError = 19,
//NSXMLParserParsedEntityRefInEpilogError = 20,
//NSXMLParserParsedEntityRefInInternalSubsetError = 21,
//NSXMLParserEntityReferenceWithoutNameError = 22,
//NSXMLParserEntityReferenceMissingSemiError = 23,
//NSXMLParserParsedEntityRefNoNameError = 24,
//NSXMLParserParsedEntityRefMissingSemiError = 25,
//NSXMLParserUndeclaredEntityError = 26,
//NSXMLParserUnparsedEntityError = 28,
//NSXMLParserEntityIsExternalError = 29,
//NSXMLParserEntityIsParameterError = 30,
//NSXMLParserUnknownEncodingError = 31,
//NSXMLParserEncodingNotSupportedError = 32,
//NSXMLParserStringNotStartedError = 33,
//NSXMLParserStringNotClosedError = 34,
//NSXMLParserNamespaceDeclarationError = 35,
//NSXMLParserEntityNotStartedError = 36,
//NSXMLParserEntityNotFinishedError = 37,
//NSXMLParserLessThanSymbolInAttributeError = 38,
//NSXMLParserAttributeNotStartedError = 39,
//NSXMLParserAttributeNotFinishedError = 40,
//NSXMLParserAttributeHasNoValueError = 41,
//NSXMLParserAttributeRedefinedError = 42,
//NSXMLParserLiteralNotStartedError = 43,
//NSXMLParserLiteralNotFinishedError = 44,
//NSXMLParserCommentNotFinishedError = 45,
//NSXMLParserProcessingInstructionNotStartedError = 46,
//NSXMLParserProcessingInstructionNotFinishedError = 47,
//NSXMLParserNotationNotStartedError = 48,
//NSXMLParserNotationNotFinishedError = 49,
//NSXMLParserAttributeListNotStartedError = 50,
//NSXMLParserAttributeListNotFinishedError = 51,
//NSXMLParserMixedContentDeclNotStartedError = 52,
//NSXMLParserMixedContentDeclNotFinishedError = 53,
//NSXMLParserElementContentDeclNotStartedError = 54,
//NSXMLParserElementContentDeclNotFinishedError = 55,
//NSXMLParserXMLDeclNotStartedError = 56,
//NSXMLParserXMLDeclNotFinishedError = 57,
//NSXMLParserConditionalSectionNotStartedError = 58,
//NSXMLParserConditionalSectionNotFinishedError = 59,
//NSXMLParserExternalSubsetNotFinishedError = 60,
//NSXMLParserDOCTYPEDeclNotFinishedError = 61,
//NSXMLParserMisplacedCDATAEndStringError = 62,
//NSXMLParserCDATANotFinishedError = 63,
//NSXMLParserMisplacedXMLDeclarationError = 64,
//NSXMLParserSpaceRequiredError = 65,
//NSXMLParserSeparatorRequiredError = 66,
//NSXMLParserNMTOKENRequiredError = 67,
//NSXMLParserNAMERequiredError = 68,
//NSXMLParserPCDATARequiredError = 69,
//NSXMLParserURIRequiredError = 70,
//NSXMLParserPublicIdentifierRequiredError = 71,
//NSXMLParserLTRequiredError = 72,
//NSXMLParserGTRequiredError = 73,
//NSXMLParserLTSlashRequiredError = 74,
//NSXMLParserEqualExpectedError = 75,
//NSXMLParserTagNameMismatchError = 76,
//NSXMLParserUnfinishedTagError = 77,
//NSXMLParserStandaloneValueError = 78,
//NSXMLParserInvalidEncodingNameError = 79,
//NSXMLParserCommentContainsDoubleHyphenError = 80,
//NSXMLParserInvalidEncodingError = 81,
//NSXMLParserExternalStandaloneEntityError = 82,
//NSXMLParserInvalidConditionalSectionError = 83,
//NSXMLParserEntityValueRequiredError = 84,
//NSXMLParserNotWellBalancedError = 85,
//NSXMLParserExtraContentError = 86,
//NSXMLParserInvalidCharacterInEntityError = 87,
//NSXMLParserParsedEntityRefInInternalError = 88,
//NSXMLParserEntityRefLoopError = 89,
//NSXMLParserEntityBoundaryError = 90,
//NSXMLParserInvalidURIError = 91,
//NSXMLParserURIFragmentError = 92,
//NSXMLParserNoDTDError = 94,
//NSXMLParserDelegateAbortedParseError = 512
