//
//  ServerItemTableViewControllerR2.m
//  BBSApp
//
//  Created by Kiwoon Sung on 8/1/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <SystemConfiguration/SystemConfiguration.h>

#import "Contants.h"

#import "ServerItemTableViewControllerR2.h"

#import "BBSAppAppDelegate.h"
#import "BBSItemLocation.h"

#import "ItemInformationTableViewCell.h"
#import "ItemViewController.h"

#import "ItemXMLReader.h"
#import "ItemInformationXMLReader.h"

// Imports for the internal Web server
#import "defs.h"

@implementation ServerItemTableViewControllerR2

@synthesize list;
@synthesize tableViewMember;

@synthesize listItemInformation; // THIS is a mutable array list keeping the item information objects acquired from the server
@synthesize activityIndicatorView = _activityIndicatorView;
@synthesize itemViewController;
@synthesize receivedData = _receivedData;


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		// Initialization code
        self.title = @"Repository";
        
        // Setup the tab bar image: Temporary use since it's for Downloads
		self.tabBarItem.image = [UIImage imageNamed:@"repository.png"]; //UITabBarSystemItemDownloads; //
		
		// set the long name shown in the navigation bar
		self.navigationItem.title=@"Bulletin Board";
        
        // Testing
        list = [[NSMutableArray alloc]initWithCapacity:2];
        [self generateTableEntries];
        
        // create a custom navigation bar button and set it to always say "back"
		UIBarButtonItem *temporaryBarButtonItem=[[UIBarButtonItem alloc] init];
		temporaryBarButtonItem.title=@"Back";
		self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
		[temporaryBarButtonItem release];
        
        // Start composing the menu list with the appropriate indicator
        // IF NEEDED, spawn a thread to fetch the item list data so that the UI is not blocked while
        // the application connects to the central repository and parses the XML file received.
        // FYI: [NSThread detachNewThreadSelector:@selector(getItemListData toTarget:self withObject:nil];
        [self showActivityIndicator];
        
        didLaunchHTTPDaemon = NO;
        
        //[self getItemListData]; // for testing
//        [self generateItemInformationListByXML];
	}
	return self;
}


- (void)loadView {
    //    CGRect screenRect = [[UIScreen mainScreen] applicationFrame];
	
	// setup our parent content view and embed it to your view controller
	//
	UIView *contentView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
    contentView.autoresizesSubviews = YES;
	contentView.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
	contentView.backgroundColor = [UIColor groupTableViewBackgroundColor];	// use the table view background color
	self.view = contentView;
	[contentView release];
	
	// create the UIToolbar at the bottom of the view controller
	//
	toolbar = [UIToolbar new];
	toolbar.barStyle = UIBarStyleBlackTranslucent;
	
	// size up the toolbar and set its frame
	[toolbar sizeToFit];
	CGFloat toolbarHeight = [toolbar frame].size.height - 20.0;
	CGRect mainViewBounds = self.view.bounds;
    //    CGFloat toolbarYOrigin = self.navigationController.navigationBar.frame.size.height;
    CGFloat toolbarYOrigin = 0.0;
    NSLog(@"toobarYOrigin = %f", toolbarYOrigin);
	[toolbar setFrame:CGRectMake(0.0, //CGRectGetMinX(mainViewBounds),
								 toolbarYOrigin, //CGRectGetMinY(mainViewBounds) + CGRectGetHeight(mainViewBounds) - (toolbarHeight * 2.0) + 2.0,
								 CGRectGetWidth(mainViewBounds),
								 toolbarHeight)];
	
	[self.view addSubview:toolbar];
	
    //	currentSystemItem = UIBarButtonSystemItemDone;
	[self createToolbarItems];
    
    // Then create a table view. This will contain a scroll view in itself by default
    CGRect tableViewRect = CGRectMake(0.0, 
                                      toolbarYOrigin + toolbarHeight, 
                                      CGRectGetWidth(mainViewBounds), 
                                      CGRectGetHeight(mainViewBounds) - toolbarHeight - toolbarYOrigin - 93);
    tableViewMember = [[UITableView alloc]initWithFrame:tableViewRect style:UITableViewStylePlain];
    tableViewMember.delegate = self;
    tableViewMember.dataSource = self;
    
    [self showTableFooter];
    
    [self.view addSubview:tableViewMember];
    
    NSLog(@"ServerItemTableViewControllerR2: loadView exit");
}


- (void)viewDidLoad {
    [super viewDidLoad];
    
    NSLog(@"ServerItemTableViewControllerR2: viewDidLoad");
    [self displayStatusOnToolbar:@"Hello" showIndicator:NO showStopButton: NO showRefreshButton:YES];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
    [self hideActivityIndicator];
    [self refreshTable];
    // TODO: 
    // also need to display the network activity indicator on the top of the screen
    // [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    
    // THESE are only for trial
    // send URL synchronous request
    // [self sendHTTPRequestSync];
    //[self generateItemInstanceByXMLTest];
    
    // Remove the network activity indicator.
    // refer to the above code
    
    
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}


- (void)dealloc {
	[super dealloc];
}


#pragma mark UITableView delegate methods

// decide what kind of accesory view (to the far right) we will use
- (UITableViewCellAccessoryType)tableView:(UITableView *)tableView accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellAccessoryDisclosureIndicator;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [listItemInformation count];
}

- (id)objectInListAtIndex:(NSUInteger)theIndex {
    NSLog(@"objectInListAtIndex: %d", theIndex);
    return [listItemInformation objectAtIndex:theIndex];
}

- (void)getList:(id *)objsPtr range:(NSRange)range {
    [listItemInformation getObjects:objsPtr range:range];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	//static NSString *MyIdentifier = @"MyIdentifier";
    //	
    //	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
    //	if (cell == nil) {
    //		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
    //	}
    //	// Configure the cell
    //    cell.text = [[list objectAtIndex:indexPath.row] objectForKey:@"name"];
    //	
    //
    //	return cell;
    
	static NSString *MyIdentifier = @"MyIdentifier";
    NSLog(@"tableView:cellForRowAtIndexPath:indexPath.row=%d", indexPath.row);
    
  	ItemInformationTableViewCell *cell = (ItemInformationTableViewCell *)[tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[ItemInformationTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
	}
    
    // If the information isn't accessible (which could happen if the network isn't available), show an informative
    // message in the first row of the table.
	if ([self isDataSourceAvailable] == NO) {
        cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:@"DefaultTableViewCell"] autorelease];
        cell.text = @"Host Not Available";
		cell.textColor = [UIColor colorWithWhite:0.5 alpha:0.5];
		cell.accessoryType = UITableViewCellAccessoryNone;
		return cell;
	}
	
	ItemInformation *itemInformationForRow = [listItemInformation objectAtIndex:indexPath.row];
    [cell setItemInformation:itemInformationForRow];
    
	return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    NSLog(@"ServerItemTableViewController clicked [row, section] = [%d, %d]", indexPath.row, indexPath.section);
    
	ItemInformation *itemInformation;
    itemInformation = [(ItemInformationTableViewCell *)[tableView cellForRowAtIndexPath:indexPath] getItemInformation];
    NSLog(@"indexPath.row = %d", indexPath.row);
    NSLog(@"selected.itemName = %@", itemInformation.itemName);
    NSLog(@"selected.fileName = %@", itemInformation.fileName);
    
    
    // Only the item information intance which is supposed to contain only the brief information about the BBS
    // item is stored in the cell instance for the table.
    // Therefore, downloading the corresponding xml file (the name is given in the item information instance
    // should be done before passing the contents to the item view (table type) controller
    ItemViewController *localItemViewController = [[ItemViewController alloc] initWithItemFileName:itemInformation.fileName itemLocation:BBSItemLocationServer];
    // Here, set the link information to get the contents.
    // According to the UI guideline, contents are loaded only after the view shows up.
    //    [itemViewController loadItem:itemInformation.fileName]; // only file name is needed
    //    itemTableViewController.itemInformation = itemInformation;
    
    self.itemViewController = localItemViewController;
    [[self navigationController] pushViewController:self.itemViewController animated:YES];
    [localItemViewController release];
    //    return nil;
    
    // Refer
    // deselect the new row using animation
    //    [tableView deselectRowAtIndexPath:newIndexPath animated:YES];
    //	
    //	// get the element that is represented by the selected row.
    //	AtomicElement *element = [dataSource atomicElementForIndexPath:newIndexPath];
    //	
    //	// create an AtomicElementViewController. This controller will display the full size tile for the element
    //	AtomicElementViewController *elementController = [[AtomicElementViewController alloc] init];
    //    
    //	// set the element for the controller
    //	elementController.element = element;
    //	
    //	// push the element view controller onto the navigation stack to display it
    //	[[self navigationController] pushViewController:elementController animated:YES];
    //	[elementController release];
    
    
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
/*    BBSAppAppDelegate *appDelegate = [[UIApplication sharedApplication]delegate];
    NSString *hostName = [[NSString alloc]initWithString:[appDelegate getServerHostName]];
  */  
    //return [@"Notices in server:%@" stringByAppendingString:hostName];
    return @"Notice in server";
}

/*
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
 
 if (editingStyle == UITableViewCellEditingStyleDelete) {
 }
 if (editingStyle == UITableViewCellEditingStyleInsert) {
 }
 }
 */
/*
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
 return YES;
 }
 */
/*
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
 }
 */
/*
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
 return YES;
 }
 */

#pragma mark UITableViewDelegate Ends


#pragma mark NSURLConnection delegate


- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse 
                                                                     *)response 
{ 
    // this method is called when the server has determined that it 
    // has enough information to create the NSURLResponse 
    // it can be called multiple times, for example in the case of a 
    // redirect, so each time we reset the data. 
    // receivedData is declared as a method instance elsewhere 
    [self.receivedData setLength:0]; 
} 


- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // append the new data to the receivedData
    // receivedData is declared as a method instance elsewhere
    [self.receivedData appendData:data];
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    [connection release];
    // receivedData is declared as a method instance elsewhere
    [self.receivedData release];
    
    // inform the user
    NSLog(@"Connection failed! Error - %@ %@",
          [error localizedDescription],
          [[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
    
    [self displayStatusOnToolbar:@"Fail with error" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    // receivedData is declared as a method instance elsewhere
    NSLog(@"Succeeded! Received %d bytes of data",[self.receivedData length]);
    
    [self reloadItemInformationList:self.receivedData];
    
    // release the connection, and the data object
    [connection release];
    [self.receivedData release];
    
    [tableViewMember reloadData];
    NSDate *now = [[NSDate alloc] init];
    NSString *label = @"";
    label = [label stringByAppendingFormat:@"Updated on %@", [now description]];
    [self displayStatusOnToolbar:label showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


#pragma mark NSURLConnection delegate End

#pragma mark User defined functions

- (void)createToolbarItems
{	
	// create the system-defined "OK or Done" button
//    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc]
//                                   initWithBarButtonSystemItem:UIBarButtonSystemItemDone 
//                                   target:nil action:nil];
    CGRect infoRect = CGRectMake(0.0, 0.0, 150, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(1.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = @"Latest update "; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// create a refresh tool bar button for future use
	UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                 target:self 
                                                                                 action:@selector(refreshAction:)];
	
    // Create an activity indicator at the rightmost of the toolbar
    UIActivityIndicatorView *activityIndicator;
    CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
    activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
    activityIndicator.opaque = YES;
    activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
    [activityIndicator startAnimating];
    
    UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
    [activityIndicator release];
	
	NSArray *items = [NSArray arrayWithObjects: indicatorItem, systemItem, flexItem, refreshItem, nil];
	[toolbar setItems:items animated:YES];

	[systemItem release];
	[flexItem release];
	[refreshItem release];
}


- (void)refreshTable {
    [self displayStatusOnToolbar:@"Refreshing" showIndicator:YES showStopButton:YES showRefreshButton:NO];
    
    [self generateItemListInformationByURL];

}


- (void)refreshAction:(id)sender {
    NSLog(@"refresh button clicked");
    [self displayStatusOnToolbar:@"Refreshing" showIndicator:YES showStopButton:YES showRefreshButton:NO];
    
    // Trial
    // [self launchHTTPDaemon];
   
    // int system (const char *command) test
    int i;
    i=system ("echo");
    printf ("The value returned was: %d.\n",i);    
    
//    [self checkCommandsAvailability];
    
    // You need to refresh and control the activity indicator in the tool bar here
    // Also need to control time-out
    
    // Trial: By URL loading, request item list information to the server, named "BBS_REP"
    [self generateItemListInformationByURL];
}


- (void)stopAction:(id)sender {
    NSLog(@"stop button clicked");
    
    [self displayStatusOnToolbar:@"Stopped" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)generateTableEntries {
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"iPhone For Sale", @"itemName",
                     nil]];
    
    [list addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                     @"BBS Item For Testing", @"itemName",
                     nil]];
    
}    


- (void) showActivityIndicator {
    // Show the activity indicator before parsing the contents.
    UIActivityIndicatorView *localActivityIndicatorView;
    CGRect frame = CGRectMake(kActivityIndicatorXOrigin, kActivityIndicatorYOrigin, kActivityIndicatorSize, kActivityIndicatorSize);
    localActivityIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:frame];
    [localActivityIndicatorView startAnimating];
    localActivityIndicatorView.hidesWhenStopped = YES;
    localActivityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleGray;
    [localActivityIndicatorView sizeToFit];
    localActivityIndicatorView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
                                                   UIViewAutoresizingFlexibleRightMargin |
                                                   UIViewAutoresizingFlexibleTopMargin |
                                                   UIViewAutoresizingFlexibleBottomMargin);
    
    self.activityIndicatorView = localActivityIndicatorView;
    
    [self.view addSubview:self.activityIndicatorView];
    
}


- (void)hideActivityIndicator {
    [self.activityIndicatorView stopAnimating];
    
}


- (void)showTableHeader {
    // See if anything worthy
    // trail header
    CGRect titleRect = CGRectMake(0, 50, 300, 40); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor blueColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont boldSystemFontOfSize:18]; 
    tableTitle.text = @"Name"; 
    
    tableViewMember.tableFooterView = tableTitle;
    
    //[self.tableView reloadData]; 
    [tableTitle release];
    
}    


- (void)showTableFooter {
    // See if anything worthy
    CGRect titleRect = CGRectMake(0, 50, 300, 45); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor lightGrayColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont fontWithName:@"Arial" size:11]; 
    tableTitle.text = @"Copyright 2008 by Columbia University";
    tableTitle.textAlignment = UITextAlignmentCenter;
    
    tableViewMember.tableFooterView = tableTitle;
    
    [tableTitle release];
    
}    


- (BOOL)isDataSourceAvailable {
    static BOOL checkNetwork = YES;
    if (checkNetwork) { // Since checking the reachability of a host can be expensive, cache the result and perform the reachability check once.
        checkNetwork = NO;
        
        Boolean success;    
        const char *host_name = "www.apple.com"; // THIS IS FOR TESTING! Change the value acquired from ZeronConf later
        
        SCNetworkReachabilityRef reachability = SCNetworkReachabilityCreateWithName(NULL, host_name);
        SCNetworkReachabilityFlags flags;
        success = SCNetworkReachabilityGetFlags(reachability, &flags);
        _isDataSourceAvailable = success && (flags & kSCNetworkFlagsReachable) && !(flags & kSCNetworkFlagsConnectionRequired);
        CFRelease(reachability);
    }
    return _isDataSourceAvailable;
}


- (void)generateItemInstanceByXMLTest {
    NSError *parseError = nil;
    NSLog(@"generateItemInstanceByXMLTest");
    
    ItemXMLReader *streamingParser = [[ItemXMLReader alloc] init];
    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:@"http://192.168.1.4/iPhoneSale.xml"] parseError:&parseError];
    [streamingParser release];   
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
}


- (void)generateItemInformationListByXML {
    NSError *parseError = nil;
    NSLog(@"generateItemInformationListByXML called");
    
    // Retrieve the server information
    // Set up the cell.
	BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    // This is the code written before the modification on the server side.
    // Should request the item information list to the server first,
    // and then this will be able to get the file like ItemInformationListFromServer.xml.
    NSString *URLString = [appDelegate getServerURLString];
    URLString = [URLString stringByAppendingString:@"ItemInformationListFromServer.xml"];
    NSLog(@"try to get the item information from \"%@\"", URLString);
    
    
    ItemInformationXMLReader *streamingParser = [[ItemInformationXMLReader alloc] init];
    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    
    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
            case NSXMLParserPrematureDocumentEndError:
                NSLog(@"NSXMLParserPrematureDocumentEndError");
                // See the end of this source to check error types.
                // The corresponding actions should be addressed here soon.
                break;
            default:
                NSLog(@"error code = %d", [parseError code]);
        }
    }
    else {
        NSLog(@"Item information XML has been parsed correctly");
        //self.listItemInformation = [[NSMutableArray alloc] initWithArray:[streamingParser getListItemInformation] copyItems:YES];
        self.listItemInformation = [streamingParser getListItemInformation];        
    }
    
    [streamingParser release];   
    
    NSLog(@"self.listItemInformation has %d item information", [self.listItemInformation count]);
    
    // debug info
    NSUInteger count = 0;
    for (ItemInformation *itemInfo in self.listItemInformation) {
        NSLog(@"count: %d", count++);
        NSLog(@"itemName: %@", itemInfo.itemName);
        NSLog(@"shortDes: %@", itemInfo.shortDescription);
    }
}


// URL in this method should be acquired from the server information class
// Don't need to pass URL information to this method without a reason
- (void)generateItemListInformationByURL {

//    NSError *parseError = nil;
    NSLog(@"generateItemListInformationByURL called");
    
    // Retrieve the server information
    BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    if (appDelegate.isServerFound == NO) {
        NSLog(@"the central repository is not available at this moment");
        [self displayStatusOnToolbar:@"Server is not running" showIndicator:NO showStopButton:NO showRefreshButton:YES];
        return;
    } else {
        NSLog(@"repository found");
    }
    
    NSString *URLString = [appDelegate getServerConnectionInformation];
    
    // Bug fix: If no network connection, URLString could be null;
    if ((URLString == nil) || ([URLString isEqualToString:@""])) {
        return;
    }
    
    // TEST
    //URLString = @"http://192.168.1.4:52368/";
    
    // Need to specify a post method with the URI /BBS and "command=itemListInformation" (case-insensitive)

    NSLog(@"try to get the item information from \"%@\"", URLString);
    
    NSString *post = @"registered_name=test&command=iteminformationlist";
    NSData *postData = [post dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
    
    NSString *postLength = [NSString stringWithFormat:@"%d", [postData length]];
    
    NSMutableURLRequest *request = [[[NSMutableURLRequest alloc] init] autorelease];
    URLString = [URLString stringByAppendingString:@"BBS"];
    URLString = [URLString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    [request setURL:[NSURL URLWithString:URLString]];
    [request setHTTPMethod:@"POST"];
    [request setValue:postLength forHTTPHeaderField:@"Content-Length"];
    [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPBody:postData];
    [request setTimeoutInterval:_REQUEST_TIME_OUT_IN_SEC];
    [request setCachePolicy:NSURLRequestUseProtocolCachePolicy];

    // ..
    // create the connection with the request
    // and start loading the data
    
    //NSURLConnection *theConnection=[[NSURLConnection alloc] initWithRequest:request delegate:self];
    
    // Synchronous send
    /*
     NSURLResponse **response = nil;
     NSError **error = nil;
    NSLog(@"request sent");
    NSData *responseData = [NSURLConnection sendSynchronousRequest:request returningResponse:response error:error];
    
    NSString *responseString = [[NSString alloc]initWithData:responseData encoding:NSUTF8StringEncoding];
    NSLog(@"response came:%@",  responseString);
*/
    
    // Asynchronous send
    NSURLConnection *theConnection=[[NSURLConnection alloc] initWithRequest:request 
                                                                   delegate:self]; 

    if (theConnection) { 
        self.receivedData = [[NSMutableData data] retain]; 
    } else { 
        // inform the user that the download could not be made 
        NSLog(@"error on requesting");
        
        [self displayStatusOnToolbar:@"Failed to request information" showIndicator:NO showStopButton:NO showRefreshButton:YES];
    } 


}


- (void)reloadItemInformationList:(NSData *)receivedData {
    NSError *parseError = nil;

    ItemInformationXMLReader *streamingParser = [[ItemInformationXMLReader alloc] init];
    //    [streamingParser parseXMLFileAtURL:[NSURL URLWithString:URLString] parseError:&parseError];
    [streamingParser parseXMLWithData:receivedData parseError:&parseError];

    if ([[parseError domain] isEqualToString:NSXMLParserErrorDomain]) {
        switch([parseError code]) {
        case NSXMLParserPrematureDocumentEndError:
        NSLog(@"NSXMLParserPrematureDocumentEndError");
        // See the end of this source to check error types.
        // The corresponding actions should be addressed here soon.
        break;
        default:
        NSLog(@"error code = %d", [parseError code]);
        }
    }
    else {
        NSLog(@"Item information XML has been parsed correctly");
        //self.listItemInformation = [[NSMutableArray alloc] initWithArray:[streamingParser getListItemInformation] copyItems:YES];
        self.listItemInformation = [streamingParser getListItemInformation];        
    }

    [streamingParser release];   

    NSLog(@"self.listItemInformation has %d item information", [self.listItemInformation count]);

    // debug info
    NSUInteger count = 0;
    for (ItemInformation *itemInfo in self.listItemInformation) {
        NSLog(@"count: %d", count++);
        NSLog(@"fileName: %@", itemInfo.fileName);
        NSLog(@"itemName: %@", itemInfo.itemName);
        NSLog(@"shortDes: %@", itemInfo.shortDescription);
    }
}


- (void)displayStatusOnToolbar:(NSString *)text 
                 showIndicator:(BOOL)showIndicator
                showStopButton:(BOOL)showStopButton
             showRefreshButton:(BOOL)showRefreshButton 
{
    CGRect infoRect = CGRectMake(0.0, 0.0, STATUS_ON_TOOLBAR_WIDTH, 13); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(0.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = text; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// Create and add an indicator if needed
	NSMutableArray *items = [[NSMutableArray alloc]init];
    if (showIndicator) {
        // Create an activity indicator at the rightmost of the toolbar
        UIActivityIndicatorView *activityIndicator;
        CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
        activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
        activityIndicator.opaque = YES;
        activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
        [activityIndicator startAnimating];
        
        UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
        
        [items addObject:indicatorItem];
        
        [activityIndicator release];
    }
    
    // Mandatory field: add systemItem
    [items addObject:systemItem];
    [items addObject:flexItem];
    
    // Create and add a stop button
    if (showStopButton) {
        // Create a stop tool bar button
        UIBarButtonItem *stopItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop 
                                                                                  target:self 
                                                                                  action:@selector(stopAction:)];
        
        [items addObject:stopItem];
        [stopItem release];
    }
    
    // Create and add a refresh button if needed
    if (showRefreshButton) {
        // Create a refresh tool bar button for future use
        UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                     target:self 
                                                                                     action:@selector(refreshAction:)];
        
        [items addObject:refreshItem];
        [refreshItem release];
    }
    
    // Construct the tool bar
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
}


- (void)runHTTPService:(id)sender {
    NSLog(@"thread: starting http daemong");
    
    struct shttpd_ctx	*ctx;
    
    //#if !defined(NO_AUTH)
    //    NSLog(@"NO_AUTH");
    //	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'A') {
    //		if (argc != 6)
    //			usage(argv[0]);
    //		exit(edit_passwords(argv[2],argv[3],argv[4],argv[5]));
    //	}
    //#endif /* NO_AUTH */
    
    //	if (argc == 2 && (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help")))
    //		usage(argv[0]);
    
#ifndef _WIN32
    NSLog(@"ndef _WIN32");
	(void) signal(SIGCHLD, signal_handler);
	(void) signal(SIGPIPE, SIG_IGN);
#endif /* _WIN32 */
    
	(void) signal(SIGTERM, signal_handler);
	(void) signal(SIGINT, signal_handler);
    
	//ctx = shttpd_init(argc, argv);
    ctx = shttpd_init(0, nil);
    
	elog(E_LOG, NULL, "shttpd %s started on port(s) %s, serving %s",
         VERSION, ctx->options[OPT_PORTS], ctx->options[OPT_ROOT]);
    
	//while (exit_flag == 0)
    while (1)
		shttpd_poll(ctx, 10 * 1000);
    
    //	elog(E_LOG, NULL, "Exit on signal %d", exit_flag);
	shttpd_fini(ctx);
    
    //    return (EXIT_SUCCESS);
}


- (void)launchHTTPDaemon {
    if (!didLaunchHTTPDaemon) {
        // SHTTPD is basically based on polling-based listening to the port assigned
        // Need to create a thread to run the polling
        
        // Refer:[NSThread detachNewThreadSelector:@selector(getItemListData toTarget:self withObject:nil];
        
        /*NSThread* myThread = [[NSThread alloc] initWithTarget:self
                                                     selector:@selector(runHTTPService:)
                                                       object:nil];
        [myThread start];
        */
        didLaunchHTTPDaemon = YES;
        [NSThread detachNewThreadSelector:@selector(runHTTPService:) toTarget:self withObject:nil];
        
    }
}

static int	exit_flag;	/* Program termination flag	*/

static void
signal_handler(int sig_num)
{
	switch (sig_num) {
#ifndef _WIN32
        case SIGCHLD:
            while (waitpid(-1, &sig_num, WNOHANG) > 0) ;
            break;
#endif /* !_WIN32 */
        default:
            exit_flag = sig_num;
            break;
	}
}


- (void)checkCommandsAvailability {
    // Seems that zsh is included
    NSLog(@"checkCommandsAvailability");
    
    int _ls = system("ls");
    int _echo = system("echo");
    int _times = system("times");
    int _cd = system("cd");
    int _print = system("print test");
    int _pwd = system("_pwd");
    int _dirs = system("_dirs");
//    NSLog(@"shttpd: %d", system("./shttpd"));
    
    printf("ls: %d", _ls);
    NSLog(@"echo: %d", _echo);
    NSLog(@"times: %d", _times);
    NSLog(@"cd: %d", _cd);
    NSLog(@"print: %d", _print);
    NSLog(@"pwd: %d", _pwd);
    NSLog(@"dirs: %d", _dirs);
}

#pragma mark User defined methods END


@end
