//
//  ServiceInformationTableViewCell.m
//  BBSApp
//
//  Created by Kiwoon Sung on 8/3/08.
//  Copyright 2008 Columbia University. All rights reserved.
//
#import "Contants.h"

#import "ServiceInformationTableViewCell.h"
#import "ServiceInformation.h"

@interface ServiceInformationTableViewCell()
- (UILabel *)newLabelWithPrimaryColor:(UIColor *)primaryColor selectedColor:(UIColor *)selectedColor fontSize:(CGFloat)fontSize bold:(BOOL)bold;
@end


@implementation ServiceInformationTableViewCell

@synthesize serviceInformation = _serviceInformation;
@synthesize registeredNameLabel = _registeredNameLabel;
@synthesize connectionInformationLabel = _connectionInformationLabel;
@synthesize itemCountLabel = _itemCountLabel;

- (id)initWithFrame:(CGRect)frame reuseIdentifier:(NSString *)reuseIdentifier {
	if (self = [super initWithFrame:frame reuseIdentifier:reuseIdentifier]) {
		// Initialization code
        UIView *myContentView = self.contentView;
        
        // A label that displays the item name of the item.
        self.registeredNameLabel = [self newLabelWithPrimaryColor:[UIColor blackColor] selectedColor:[UIColor whiteColor] fontSize:ITEM_NAME_FONT_SIZE bold:YES];
        self.registeredNameLabel.textAlignment = UITextAlignmentLeft;
        self.registeredNameLabel.backgroundColor = [UIColor clearColor];
        self.registeredNameLabel.opaque = YES;
        [myContentView addSubview:self.registeredNameLabel];
        [self.registeredNameLabel release];
        
        // A label that displays the short description of the item
        // This information is only available on the item table view,
        // not in the item view
        self.connectionInformationLabel = [self newLabelWithPrimaryColor:[UIColor grayColor] selectedColor:[UIColor lightGrayColor] fontSize:SHORT_DESCRIPTION_FONT_SIZE bold:NO];
        self.connectionInformationLabel.textAlignment = UITextAlignmentLeft;
        self.connectionInformationLabel.backgroundColor = [UIColor clearColor];
        [myContentView addSubview:self.connectionInformationLabel];
        [self.connectionInformationLabel release];
        
        // A label that displays the number of items contained in the service
        self.itemCountLabel = [self newLabelWithPrimaryColor:[UIColor blueColor] selectedColor:[UIColor lightGrayColor] fontSize:ITEM_COUNT_FONT_SIZE bold:NO];
        self.itemCountLabel.textAlignment = UITextAlignmentLeft;
        self.itemCountLabel.backgroundColor = [UIColor clearColor];
        [myContentView addSubview:self.itemCountLabel];
        [self.itemCountLabel release];
	}
	return self;
}


- (void)dealloc {
    [self.serviceInformation release];
    [self.registeredNameLabel release];
    [self.connectionInformationLabel release];
    [self.itemCountLabel release];
    
    [super dealloc];
}


- (ServiceInformation *)getServiceInformation {
    return self.serviceInformation;
}


- (void)set:(ServiceInformation *)newServiceInformation {
    NSLog(@"set: newServiceInformation");

    [newServiceInformation retain];
    [self.serviceInformation release];
    self.serviceInformation = newServiceInformation;
    
    // MEMO:
    // The following one line could lead infinite loop since it is symthesized setter itself
    // so calls the setServiceInformation recursively, provided that this function name is
    // "setServiceInformation". It seems that this means setter function by naming convention.
    // self.serviceInformation = [[ServiceInformation alloc]initWithService:newServiceInformation];
    
    NSLog(@"registered name = %@", [newServiceInformation getServiceName]);

    // Trial: change the color for the label when it's for "BBS_REP"
    if ([[newServiceInformation getServiceName] isEqualToString:@"BBS_REP"]) {
        self.registeredNameLabel.text = @"Server";
    } else {
        self.registeredNameLabel.text = [newServiceInformation getServiceName];
    }
    
    int count = [newServiceInformation count];
    NSString *connectionInfo = [[NSString alloc]initWithFormat:@"%d", [newServiceInformation count]];
    if (count == 0) {
        connectionInfo = @"No item in ";
    } else if (count == 1) {
        connectionInfo = [connectionInfo stringByAppendingString:@" item in "];
    } else if (count > 1) {
        connectionInfo = [connectionInfo stringByAppendingString:@" items in "];
    }

    connectionInfo = [connectionInfo stringByAppendingString:[newServiceInformation.netService hostName]];
    connectionInfo = [connectionInfo stringByAppendingString:@":"];
    connectionInfo = [connectionInfo stringByAppendingFormat:@"%d", [newServiceInformation.netService port]];
    self.connectionInformationLabel.text = connectionInfo;
    
    // hart to place this field on the cell. remove it. instead, connection info has this information
//    NSString *itemCount = [[NSString alloc]initWithFormat:@"%d item(s)", [newServiceInformation.items count]];
    // self.itemCountLabel.text = itemCount;
    
//    [self setNeedsDisplay];
}



- (void)layoutSubviews {
    
    [super layoutSubviews];
    CGRect contentRect = self.contentView.bounds;
    
    // In this example we will never be editing, but this illustrates the appropriate pattern
    if (!self.editing) {
        
        CGFloat boundsX = contentRect.origin.x;
        CGRect frame;
        
        // Place the registered name label.
        frame = CGRectMake(boundsX + LEFT_COLUMN_OFFSET, UPPER_ROW_TOP, LEFT_COLUMN_WIDTH, 20);
        self.registeredNameLabel.frame = frame;
        
        // Place the connection information label.
        frame = CGRectMake(boundsX + LEFT_COLUMN_OFFSET, LOWER_ROW_TOP, LEFT_COLUMN_WIDTH, 18);
        self.connectionInformationLabel.frame = frame;     
        
        // Place the item count information label
        frame = CGRectMake(boundsX + LEFT_COLUMN_OFFSET, LOWER_ROW_TOP, LEFT_COLUMN_WIDTH + MIDDLE_COLUMN_WIDTH, 30);
        
        self.itemCountLabel.frame = frame;
    }
}



- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    
	[super setSelected:selected animated:animated];
    
	// Configure the view for the selected state
    UIColor *backgroundColor = nil;
    if (selected) {
        backgroundColor = [UIColor clearColor];
    } else {
        backgroundColor = [UIColor clearColor];
    }
    
    self.registeredNameLabel.backgroundColor = backgroundColor;
    self.registeredNameLabel.highlighted = selected;
    self.registeredNameLabel.opaque = !selected;
    
    self.connectionInformationLabel.backgroundColor = backgroundColor;
    self.connectionInformationLabel.highlighted = selected;
    self.connectionInformationLabel.opaque = !selected;     
    
    self.itemCountLabel.backgroundColor = backgroundColor;
    self.itemCountLabel.highlighted = selected;
    self.itemCountLabel.opaque = !selected;     
}


- (UILabel *)newLabelWithPrimaryColor:(UIColor *)primaryColor selectedColor:(UIColor *)selectedColor fontSize:(CGFloat)fontSize bold:(BOOL)bold
{
    UIFont *font;
    if (bold) {
        font = [UIFont boldSystemFontOfSize:fontSize];
    } else {
        font = [UIFont systemFontOfSize:fontSize];
    }
    
    /*
     Views are drawn most efficiently when they are opaque and do not have a clear background, so set these defaults.  To show selection properly, however, the views need to be transparent (so that the selection color shows through).  This is handled in setSelected:animated:.
     */
	UILabel *newLabel = [[UILabel alloc] initWithFrame:CGRectZero];
	newLabel.backgroundColor = [UIColor clearColor];
	newLabel.opaque = YES;
	newLabel.textColor = primaryColor;
	newLabel.highlightedTextColor = selectedColor;
	newLabel.font = font;
	
	return newLabel;
}

@end
