//
//  ServiceTableViewController.m
//  BBSApp
//
//  Created by Ki Woon Sung on 7/25/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import <SystemConfiguration/SystemConfiguration.h>

#import "Contants.h"

#import "BBSAppAppDelegate.h"
#import "ServiceTableViewController.h"

#import "ServiceInformation.h"
#import "ServiceInformationTableViewCell.h"


@implementation ServiceTableViewController

@synthesize tableViewMember;
@synthesize listItemInformation;
@synthesize activityIndicatorView = _activityIndicatorView;
@synthesize peerItemTableViewController;


- (id)initWithStyle:(UITableViewStyle)style {
	if (self = [super initWithStyle:style]) {
		self.title = @"Peer";
		self.tabBarItem.image = [UIImage imageNamed:@"peer.png"];
//        self.tabBarItem = [[UITabBarItem alloc]initWithTabBarSystemItem:UITabBarSystemItemMostViewed tag:0];
        [self.tabBarItem setTitle:@"Around"];
		
		// set the long name shown in the navigation bar
		self.navigationItem.title=@"Nodes Around You";
		
		// create a custom navigation bar button and set it to always say "back"
		UIBarButtonItem *temporaryBarButtonItem=[[UIBarButtonItem alloc] init];
		temporaryBarButtonItem.title=@"Back";
		self.navigationItem.backBarButtonItem = temporaryBarButtonItem;
		[temporaryBarButtonItem release];		

	}
	return self;
}


- (void)dealloc {
    [self.tableViewMember release];
    [self.listItemInformation release];
    [self.activityIndicatorView release];
    [self.peerItemTableViewController release];
    
	[super dealloc];
}


- (void)loadView {
    UIView *contentView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];
    contentView.autoresizesSubviews = YES;
	contentView.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
	contentView.backgroundColor = [UIColor groupTableViewBackgroundColor];	// use the table view background color
	self.view = contentView;
	[contentView release];
	
	// create the UIToolbar at the bottom of the view controller
	toolbar = [UIToolbar new];
	toolbar.barStyle = UIBarStyleBlackTranslucent;
	
	[toolbar sizeToFit];
	CGFloat toolbarHeight = [toolbar frame].size.height - 20.0;
	CGRect mainViewBounds = self.view.bounds;
    CGFloat toolbarYOrigin = 0.0;
    NSLog(@"toobarYOrigin = %f", toolbarYOrigin);
	[toolbar setFrame:CGRectMake(0.0, 
								 toolbarYOrigin, 
								 CGRectGetWidth(mainViewBounds),
								 toolbarHeight)];
	
	[self.view addSubview:toolbar];
    
    // Create a table view. This will contain a scroll view in itself by default
    CGRect tableViewRect = CGRectMake(0.0, 
                                      toolbarYOrigin + toolbarHeight, 
                                      CGRectGetWidth(mainViewBounds), 
                                      CGRectGetHeight(mainViewBounds) - toolbarHeight - toolbarYOrigin - 93);
    tableViewMember = [[UITableView alloc]initWithFrame:tableViewRect style:UITableViewStylePlain];
    tableViewMember.delegate = self;
    tableViewMember.dataSource = self;
    
    [self showTableFooter];
    
    [self.view addSubview:tableViewMember];
    
    // Setup Zeroconf
    // Assign services newly found 
    self.listItemInformation = [[NSMutableArray alloc]initWithCapacity:10];
    searching = NO;
    countServices = 0;
    
    //[self commonSetup];
    NSLog(@"ServiceTableViewController: loadView exit");
    
}


#pragma mark UITableView delegate methods

- (UITableViewCellAccessoryType)tableView:(UITableView *)tableView accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath
{
	return UITableViewCellAccessoryDisclosureIndicator;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [listItemInformation count];
}

- (id)objectInListAtIndex:(NSUInteger)theIndex {
    NSLog(@"objectInListAtIndex: %d", theIndex);
    return [listItemInformation objectAtIndex:theIndex];
}

- (void)getList:(id *)objsPtr range:(NSRange)range {
    [listItemInformation getObjects:objsPtr range:range];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *MyIdentifier = @"MyIdentifier";
    NSLog(@"ServiceTableViewController: tableView:cellForRowAtIndexPath:indexPath.row=%d", indexPath.row);
    
  	ServiceInformationTableViewCell *cell = (ServiceInformationTableViewCell *)[tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[ServiceInformationTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
	}
    
    //	ServiceInformation *serviceInformationForRow = [listItemInformation objectAtIndex:indexPath.row];
    //    [cell setServiceInformation:serviceInformationForRow];
    [cell set:(ServiceInformation *)[listItemInformation objectAtIndex:indexPath.row]];
    
    NSLog(@"cellForRowAtIndexPath exit");
    
	return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    NSLog(@"ServerItemTableViewController clicked [row, section] = [%d, %d]", indexPath.row, indexPath.section);
    
    
	ServiceInformation *serviceInformation;
    serviceInformation = [(ServiceInformationTableViewCell *)[tableView cellForRowAtIndexPath:indexPath] getServiceInformation];
    NSLog(@"indexPath.row = %d", indexPath.row);
    NSLog(@"service name = %@", [serviceInformation.netService name]);
    NSLog(@"service hostName = %@", [serviceInformation.netService hostName]);
    NSLog(@"service port = %d", [serviceInformation.netService port]);
    NSLog(@"item count = %d", [serviceInformation.items count]);
    
    
    PeerItemTableViewController *localPeerItemTableViewController = [[PeerItemTableViewController alloc] initWithNetService:serviceInformation.netService];
    
    self.peerItemTableViewController = localPeerItemTableViewController;
    [[self navigationController] pushViewController:self.peerItemTableViewController animated:YES];
    [localPeerItemTableViewController release];
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	return @"BBS services around you";
}



#pragma mark UITableViewDelegate Ends

- (void)viewDidLoad {
	[super viewDidLoad];
    
    [self displayStatusOnToolbar:@"Hello" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
    
    // Load information
    [self refreshAction:self];    
}

- (void)viewWillDisappear:(BOOL)animated {
}

- (void)viewDidDisappear:(BOOL)animated {

}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}


#pragma mark NSNetServiceBrowser delegate methods

- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didFindDomain:(NSString*)domain moreComing:(BOOL)moreComing {
	NSString* tmp = [self transmogrify:domain];
    //	if (![self.domains containsObject:tmp]) [self.domains addObject:tmp];
    NSLog(@"didFindDomain:domain[%@] transmogrified:%@", domain, tmp);
    if (moreComing) NSLog(@"moreComing");
    
	// moreComing really means that there are no more messages in the queue from the Bonjour daemon, so we should update the UI.
	// When moreComing is set, we don't update the UI so that it doesn't 'flash'.
    //	if (!moreComing)
    //		[self updateUI];
}	


- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didFindService:(NSNetService *)netService moreComing:(BOOL)moreServicesComing {
    NSLog(@"netServiceBrowser:netServiceBrowser:didFindService");
    // Special Considerations
    // If the delegate chooses to resolve netService, it should retain netService and set 
    // itself as that service’s delegate. The delegate should, therefore, release that 
    // service when it receives the netServiceDidResolveAddress: or netService:
    // didNotResolve: delegate messages of the NSNetService class.
    
    [netService setDelegate:self];
    [netService resolve];
    [self.listItemInformation addObject:(ServiceInformation *)[[ServiceInformation alloc]initWithNetService:netService]];
    countServices++;
    if (moreServicesComing) {
        //NSLog(@"No more coming");
        [self displayStatusOnToolbar:@"More services will come" showIndicator:YES showStopButton:NO showRefreshButton:NO];
    } else {
        [self displayStatusOnToolbar:@"All found" showIndicator:YES showStopButton:NO showRefreshButton:NO];
        [netServiceBrowser stop];
    }

    [self updateListItemInformation];
}


- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didRemoveService:(NSNetService *)netService moreComing:(BOOL)moreServicesComing {
    NSLog(@"didRemoveService:%@ called in ServiceTableViewController", [netService name]);
    
    if (![self removeServiceByNetService:netService]) {
        NSLog(@"cannot remove service information: %@", [netService name]);
    }
    
    if (!moreServicesComing) {
        NSLog(@"No more coming");
    }
}



- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didNotSearch:(NSDictionary *)errorInfo {
    NSLog(@"didNotSearch");
    
    searching = NO;
    [self handleError:[errorInfo objectForKey:NSNetServicesErrorCode]];
}


- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didRemoveDomain:(NSString*)domain moreComing:(BOOL)moreComing {
    NSLog(@"netServiceBrowser:netServiceBrowser didRemoveDomain: moreComing");
	// moreComing really means that there are no more messages in the queue from the Bonjour daemon, so we should update the UI.
	// When moreComing is set, we don't update the UI so that it doesn't 'flash'.
    //	if (!moreComing)
    if (moreComing)
        NSLog(@"moreComing");
    //		[self updateUI];
}	


- (void)netServiceBrowserDidStopSearch:(NSNetServiceBrowser *)netServiceBrowser {
    NSLog(@"netServiceBrowserDidStopSearch");
    searching = NO;
}



- (void)netServiceBrowserWillSearch:(NSNetServiceBrowser *)netServiceBrowser {
    searching = YES;
    NSLog(@"netServiceBrowserWillSearch");
}


// NSNetService delegate methods
#pragma mark NSNetService delegate methods


- (void)netService:(NSNetService *)sender didNotPublish:(NSDictionary *)errorDict {
    NSLog(@" didNotPublish");
    
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
}


- (void)netService:(NSNetService *)sender didNotResolve:(NSDictionary *)errorDict {
    NSLog(@" didNotResolve");
    
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
}


- (void)netService:(NSNetService *)sender didUpdateTXTRecordData:(NSData *)data {
    NSLog(@"didUpdateTXTRecordData");
    
    /*unsigned char *buffer[[data length]];
     [data getBytes:buffer];
     
     // Keep in mind that the port information that can be obtained in this delegate
     // could not be accurate enough. Sometimes it is just a negative value.
     // Rather getting the port value when the service address is being resolved.
     if (data) {
     NSLog(@"data: %s FROM %@:%d", (char *)buffer, [sender name], [sender port]);
     } else
     {
     NSLog(@"data is null");
     }*/
    NSString *dataString = [[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding];
    if (([sender hostName] != nil) &&
        ([sender name] != nil) &&
        ([sender port] != -1) &&
        (data != nil) &&
        !([dataString isEqualToString:[@"" stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]])) // Need to do something here not to add an item without data
    {
        NSLog(@"-");
        NSLog(@"dataString:%@ from \"%@\"", dataString, [sender name]);
        NSLog(@"hostName:port = \"%@:%d\"", [sender hostName], [sender port]);
        
        
        // Update information about the number of items in the service
        NSString *localServiceName = @"";
        NSUInteger count = 0;
        for (ServiceInformation *oneService in listItemInformation) {
            localServiceName = [sender name];
            
            if ([localServiceName isEqualToString:[oneService getServiceName]]) {
                // Store the TXT record data in the service information object in the list
                // [oneService.items addObject:data]; // Don't need to do this.
                // Instead, use count txt record
                
                // find txt record for count
                NSDictionary *txtDic = [[NSDictionary alloc] init];
                txtDic = [NSNetService dictionaryFromTXTRecordData:data];
                @try {
                    NSString *countString = [[NSString alloc] initWithData:[txtDic objectForKey:@"count"] encoding:NSUTF8StringEncoding];
                    count = [countString integerValue];

                    NSLog(@"count in TXTRecord: %d", count);
                }
                @catch (NSException * e) {
                    NSLog(@"no value for count:%@", [e description]);
                    return;
                }
                oneService.count = count;
                // Find the cell containing this service information: how?
            }
        }
    }
    
    // Trial:
    for (ServiceInformation *oneService in listItemInformation) {
        NSLog(@"name: %@, itemCount=%d", [oneService getServiceName], [oneService.items count]);
    }
}


- (void)netServiceDidPublish:(NSNetService *)sender {
    NSLog(@" netServiceDidPublish");
}


- (void)netServiceDidResolveAddress:(NSNetService *)sender {
    NSLog(@"------------------------------------------------------------");
    NSLog(@"netServiceDidResolveAddress");
    NSLog(@"\tname: %@", [sender name]);
    NSLog(@"port: %d", [sender port]);
    NSLog(@"\tdomain : %@", [sender domain]);
    NSLog(@"\ttype : %@", [sender type]);
    
    countServices--;
    // Start monitoring
    [sender startMonitoring];
    NSLog(@"monitoring started for the service:%@[%@:%d]", [sender name], [sender hostName], [sender port]);
    
    NSLog(@"countServices: %d", countServices);
    if (countServices > 0) {
//        [self displayStatusOnToolbar:@"Resolving" showIndicator:YES showStopButton:YES showRefreshButton:NO];
        
        
    } else {
        // Reload table entries
        //        [tableViewMember reloadData];
        [tableViewMember reloadData];
        
        // Not sure if it's okay to assume that all service have been found at this moment.
        // But, seems quite fine to do so.
        NSDate *now = [[NSDate alloc] init];
        NSString *label = @"";
        label = [label stringByAppendingFormat:@"Updated on %@", [now description]];
        [self displayStatusOnToolbar:label showIndicator:NO showStopButton:NO showRefreshButton:YES];
    }
}


- (void)netServiceDidStop:(NSNetService *)sender {
    NSLog(@" netServiceDidStop");
    
    [self printItemInformationList];
}


- (void)netServiceWillPublish:(NSNetService *)sender {
    NSLog(@" netServiceWillPublish");
}


- (void)netServiceWillResolve:(NSNetService *)sender {
    NSLog(@" netServiceWillResolve");
}


#pragma mark User defined functions

- (void)refreshAction:(id)sender {
    NSLog(@"refresh button clicked");
    
    [self displayStatusOnToolbar:@"Updating" 
                   showIndicator:YES 
                  showStopButton:NO
               showRefreshButton:NO];
    
    // Call this at the end of this function since it restores the status bar
    [self updateListItemInformation];
    
    // Update Zeroconf information
    [self reloadNetServices];
    
    // Trial: Get the information of the services found on Bonjour from app delegate
    [self printTrialInformation];
    
    
}


- (void)stopAction:(id)sender {
    NSLog(@"stop button clicked");
    
    [self displayStatusOnToolbar:@"Stopped" 
                   showIndicator:NO
                  showStopButton:NO
               showRefreshButton:YES];
}


- (void)showTableFooter {
    // See if anything worthy
    CGRect titleRect = CGRectMake(0, 50, 300, 45); 
    UILabel *tableTitle = [[UILabel alloc] initWithFrame:titleRect]; 
    tableTitle.textColor = [UIColor lightGrayColor]; 
    tableTitle.backgroundColor = [UIColor clearColor]; 
    tableTitle.opaque = YES; 
    tableTitle.font = [UIFont fontWithName:@"Arial" size:11]; 
    tableTitle.text = @"Copyright 2008 by Columbia University";
    tableTitle.textAlignment = UITextAlignmentCenter;
    
    tableViewMember.tableFooterView = tableTitle;
    
    [tableTitle release];
    
}    


- (void)displayStatusOnToolbar:(NSString *)text 
                 showIndicator:(BOOL)showIndicator
                showStopButton:(BOOL)showStopButton
             showRefreshButton:(BOOL)showRefreshButton 
{
    CGRect infoRect = CGRectMake(0.0, 0.0, STATUS_ON_TOOLBAR_WIDTH, 20); 
    UILabel *infoLabel = [[UILabel alloc] initWithFrame:infoRect]; 
    infoLabel.textColor = [UIColor whiteColor]; 
    infoLabel.backgroundColor = [UIColor clearColor]; 
    infoLabel.opaque = YES; 
    infoLabel.shadowColor = [UIColor blackColor];
    [infoLabel setShadowOffset:CGSizeMake(0.0, 1.0)];
    infoLabel.font = [UIFont fontWithName:@"Arial" size:13]; 
    infoLabel.text = text; 
    
    UIBarButtonItem *systemItem = [[UIBarButtonItem alloc] initWithCustomView:infoLabel];
    [infoLabel release];
    
	systemItem.style = UIBarButtonItemStylePlain;
	
	// flex item used to separate the left groups items and right grouped items
	UIBarButtonItem *flexItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                              target:nil
                                                                              action:nil];
	
	// Create and add an indicator if needed
	NSMutableArray *items = [[NSMutableArray alloc]init];
    if (showIndicator) {
        // Create an activity indicator at the rightmost of the toolbar
        UIActivityIndicatorView *activityIndicator;
        CGRect indicatorRect = CGRectMake(0.0, 0.0, 13.0, 13.0);
        activityIndicator = [[UIActivityIndicatorView alloc]initWithFrame:indicatorRect];
        activityIndicator.opaque = YES;
        activityIndicator.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhite;
        [activityIndicator startAnimating];
        
        UIBarButtonItem *indicatorItem = [[UIBarButtonItem alloc]initWithCustomView:activityIndicator];
        
        [items addObject:indicatorItem];
        
        [activityIndicator release];
    }
    
    // Mandatory field: add systemItem
    [items addObject:systemItem];
    [items addObject:flexItem];
    
    // Create and add a stop button
    if (showStopButton) {
        // Create a stop tool bar button
        UIBarButtonItem *stopItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop 
                                                                                  target:self 
                                                                                  action:@selector(stopAction:)];
        
        [items addObject:stopItem];
        [stopItem release];
    }
    
    // Create and add a refresh button if needed
    if (showRefreshButton) {
        // Create a refresh tool bar button for future use
        UIBarButtonItem *refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                                                                     target:self 
                                                                                     action:@selector(refreshAction:)];
        
        [items addObject:refreshItem];
        [refreshItem release];
    }
    
    // Construct the tool bar
	[toolbar setItems:items animated:YES];
    
	[systemItem release];
	[flexItem release];
}


- (void)updateListItemInformation {
    // What in the instance of Bonjour
    
    
    // Assume that the action has been done
    //    [self displayStatusOnToolbar:@"Updated on xxx" showIndicator:NO showStopButton:NO showRefreshButton:YES];
}


- (void)printTrialInformation {
    BBSAppAppDelegate *appDelegate = (BBSAppAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    BonjourMonitor *bonjour = [appDelegate bonjourMonitor];
    
    NSLog(@"TEST: %@", [bonjour getMeaninglessString]);
    
    // Print TXTRecord
    NSArray *netServices = [[NSArray alloc]initWithArray:bonjour.netServices];
    for (NSNetService *service in netServices) {
        NSLog(@"--------------------------");
        NSLog(@"service: %@:%d", [service name], [service port]);
        NSData *txtRecordData = [service TXTRecordData];
        NSLog(@"txtRecord: %@", [[NSString alloc]initWithData:txtRecordData encoding:NSUTF8StringEncoding]);
        
    }
    
}


- (NSString *)getMeaninglessString {
    return @"hello";
}


- (BOOL)commonSetup {
	self.netServiceBrowser = [[[NSNetServiceBrowser alloc] init] autorelease];
	if(!self.netServiceBrowser) {
		return NO;
	}
	
	[self.netServiceBrowser setDelegate:self];
	return YES;
}

- (BOOL)searchForBrowsableDomains {
	if (![self commonSetup]) return NO;
	[self.netServiceBrowser searchForBrowsableDomains];
	return YES;
}

- (BOOL)searchForRegistrationDomains {
	if (![self commonSetup]) return NO;
	[self.netServiceBrowser searchForRegistrationDomains];	
	return YES;
}


// Error handling code
- (void)handleError:(NSNumber *)error
{
    NSLog(@"An error occurred. Error code = %d", [error intValue]);
    
    // Handle error here
}

- (NSString*) transmogrify:(NSString*)aString {
	
	NSString* tmp = [NSString stringWithString:aString];
	const char *ostr = [tmp UTF8String];
	const char *cstr = ostr;
	char *ptr = (char*) ostr;
	
	while (*cstr) {
		char c = *cstr++;
		if (c == '\\')
		{
			c = *cstr++;
			if (isdigit(cstr[-1]) && isdigit(cstr[0]) && isdigit(cstr[1]))
			{
				NSInteger v0 = cstr[-1] - '0';						// then interpret as three-digit decimal
				NSInteger v1 = cstr[ 0] - '0';
				NSInteger v2 = cstr[ 1] - '0';
				NSInteger val = v0 * 100 + v1 * 10 + v2;
				if (val <= 255) { c = (char)val; cstr += 2; }	// If valid three-digit decimal value, use it
			}
		}
		*ptr++ = c;
	}
	ptr--;
	*ptr = 0;
	return [NSString stringWithUTF8String:ostr];
}


// Stores newBrowser in the _netServiceBrowser instance variable. If _netServiceBrowser has already been set,
// this first sends it a -stop message before releasing it.
- (void)setNetServiceBrowser:(NSNetServiceBrowser*)newBrowser {
    NSLog(@"setNetServiceBrowser:newBrowser");
	[_netServiceBrowser stop];
	[newBrowser retain];
	[_netServiceBrowser release];
	_netServiceBrowser = newBrowser;
}



- (NSNetServiceBrowser*)netServiceBrowser {
	return _netServiceBrowser;
}

- (void)reloadNetServices {
    [self.netServiceBrowser stop];
    //    countServices = 0;
    
    [self.listItemInformation release];
    self.listItemInformation = [[NSMutableArray alloc]init];
    [self commonSetup];
    [self.netServiceBrowser searchForServicesOfType:@"_7dsbbs._tcp." inDomain:@""];
}


- (BOOL)removeServiceByNetService:(NSNetService *)netService {
    BOOL result = NO;
    
    for (ServiceInformation *temp in self.listItemInformation) {
        if ([[temp.netService name] isEqualToString:[netService name]]) {
            [self.listItemInformation removeObject:temp];   
            result = YES;
        }
    }   
    
    return result;
}


- (void)printItemInformationList {
    NSLog(@"------");
    for (ServiceInformation *temp in self.listItemInformation) {
        NSLog(@"name: %@", [temp.netService name]);
    }
}

@end

